# Copyright 2016 Dell Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

"""
Core backend volume driver interface.

All backend drivers should support this interface as a bare minimum.
"""

from cinder.interface import base


class VolumeDriverCore(base.CinderInterface):
    """Core backend driver required interface."""

    def do_setup(self, context):
        """Any initialization the volume driver needs to do while starting.

        Called once by the manager after the driver is loaded.
        Can be used to set up clients, check licenses, set up protocol
        specific helpers, etc.

        :param context: The admin context.
        """

    def check_for_setup_error(self):
        """Validate there are no issues with the driver configuration.

        Called after do_setup(). Driver initialization can occur there or in
        this call, but must be complete by the time this returns.

        If this method raises an exception, the driver will be left in an
        "uninitialized" state by the volume manager, which means that it will
        not be sent requests for volume operations.

        This method typically checks things like whether the configured
        credentials can be used to log in the storage backend, and whether any
        external dependencies are present and working.

        :raises VolumeBackendAPIException: in case of setup error.
        """

    def get_volume_stats(self, refresh=False):
        """Collects volume backend stats.

        The get_volume_stats method is used by the volume manager to collect
        information from the driver instance related to information about the
        driver, available and used space, and driver/backend capabilities.

        It returns a dict with the following required fields:

        * volume_backend_name
            This is an identifier for the backend taken from cinder.conf.
            Useful when using multi-backend.
        * vendor_name
            Vendor/author of the driver who serves as the contact for the
            driver's development and support.
        * driver_version
            The driver version is logged at cinder-volume startup and is useful
            for tying volume service logs to a specific release of the code.
            There are currently no rules for how or when this is updated, but
            it tends to follow typical major.minor.revision ideas.
        * storage_protocol
            The protocol used to connect to the storage, this should be a short
            string such as: "iSCSI", "FC", "nfs", "ceph", etc.
        * total_capacity_gb
            The total capacity in gigabytes (GiB) of the storage backend being
            used to store Cinder volumes. Use keyword 'unknown' if the backend
            cannot report the value or 'infinite' if there is no upper limit.
            But, it is recommended to report real values as the Cinder
            scheduler assigns lowest weight to any storage backend reporting
            'unknown' or 'infinite'.

        * free_capacity_gb
            The free capacity in gigabytes (GiB). Use keyword 'unknown' if the
            backend cannot report the value or 'infinite' if there is no upper
            limit. But, it is recommended to report real values as the Cinder
            scheduler assigns lowest weight to any storage backend reporting
            'unknown' or 'infinite'.

        And the following optional fields:

        * reserved_percentage (integer)
            Percentage of backend capacity which is not used by the scheduler.
        * location_info (string)
            Driver-specific information used by the driver and storage backend
            to correlate Cinder volumes and backend LUNs/files.
        * QoS_support (Boolean)
            Whether the backend supports quality of service.
        * provisioned_capacity_gb
            The total provisioned capacity on the storage backend, in gigabytes
            (GiB), including space consumed by any user other than Cinder
            itself.
        * max_over_subscription_ratio
            The maximum amount a backend can be over subscribed.
        * thin_provisioning_support (Boolean)
            Whether the backend is capable of allocating thinly provisioned
            volumes.
        * thick_provisioning_support (Boolean)
            Whether the backend is capable of allocating thick provisioned
            volumes. (Typically True.)
        * total_volumes (integer)
            Total number of volumes on the storage backend. This can be used in
            custom driver filter functions.
        * filter_function (string)
            A custom function used by the scheduler to determine whether a
            volume should be allocated to this backend or not. Example:

              capabilities.total_volumes < 10

        * goodness_function (string)
            Similar to filter_function, but used to weigh multiple volume
            backends. Example:

              capabilities.capacity_utilization < 0.6 ? 100 : 25

        * multiattach (Boolean)
            Whether the backend supports multiattach or not. Defaults to False.
        * sparse_copy_volume (Boolean)
            Whether copies performed by the volume manager for operations such
            as migration should attempt to preserve sparseness.
        * online_extend_support (Boolean)
            Whether the backend supports in-use volume extend or not. Defaults
            to True.

        The returned dict may also contain a list, "pools", which has a similar
        dict for each pool being used with the backend.

        :param refresh: Whether to discard any cached values and force a full
                        refresh of stats.
        :returns: dict of appropriate values (see above).
        """

    def create_volume(self, volume):
        """Create a new volume on the backend.

        This method is responsible only for storage allocation on the backend.
        It should not export a LUN or actually make this storage available for
        use, this is done in a later call.

        TODO(smcginnis): Add example data structure of volume object.

        :param volume: Volume object containing specifics to create.
        :returns: (Optional) dict of database updates for the new volume.
        :raises VolumeBackendAPIException: if creation failed.
        """

    def delete_volume(self, volume):
        """Delete a volume from the backend.

        If the driver can talk to the backend and detects that the volume is no
        longer present, this call should succeed and allow Cinder to complete
        the process of deleting the volume.

        :param volume: The volume to delete.
        :raises VolumeIsBusy: if the volume is still attached or has snapshots.
                 VolumeBackendAPIException on error.
        """

    def initialize_connection(self, volume, connector, initiator_data=None):
        """Allow connection to connector and return connection info.

        :param volume: The volume to be attached.
        :param connector: Dictionary containing information about what is being
                          connected to.
        :param initiator_data: (Optional) A dictionary of driver_initiator_data
                               objects with key-value pairs that have been
                               saved for this initiator by a driver in previous
                               initialize_connection calls.
        :returns: A dictionary of connection information. This can optionally
                  include a "initiator_updates" field.

        The "initiator_updates" field must be a dictionary containing a
        "set_values" and/or "remove_values" field. The "set_values" field must
        be a dictionary of key-value pairs to be set/updated in the db. The
        "remove_values" field must be a list of keys, previously set with
        "set_values", that will be deleted from the db.

        May be called multiple times to get connection information after a
        volume has already been attached.
        """

    def attach_volume(self, context, volume, instance_uuid, host_name,
                      mountpoint):
        """Lets the driver know Nova has attached the volume to an instance.

        :param context: Security/policy info for the request.
        :param volume: Volume being attached.
        :param instance_uuid: ID of the instance being attached to.
        :param host_name: The host name.
        :param mountpoint: Device mount point on the instance.
        """

    def terminate_connection(self, volume, connector):
        """Remove access to a volume.

        :param volume: The volume to remove.
        :param connector: The Dictionary containing information about the
                          connection. This is optional when doing a
                          force-detach and can be None.
        """

    def detach_volume(self, context, volume, attachment=None):
        """Detach volume from an instance.

        :param context: Security/policy info for the request.
        :param volume: Volume being detached.
        :param attachment: (Optional) Attachment information.
        """

    def clone_image(self, volume, image_location, image_id, image_metadata,
                    image_service):
        """Clone an image to a volume.

        :param volume: The volume to create.
        :param image_location: Where to pull the image from.
        :param image_id: The image identifier.
        :param image_metadata: Information about the image.
        :param image_service: The image service to use.
        :returns: Model updates.
        """

    def copy_image_to_volume(self, context, volume, image_service, image_id):
        """Fetch the image from image_service and write it to the volume.

        :param context: Security/policy info for the request.
        :param volume: The volume to create.
        :param image_service: The image service to use.
        :param image_id: The image identifier.
        :returns: Model updates.
        """

    def copy_volume_to_image(self, context, volume, image_service, image_meta):
        """Copy the volume to the specified image.

        :param context: Security/policy info for the request.
        :param volume: The volume to copy.
        :param image_service: The image service to use.
        :param image_meta: Information about the image.
        :returns: Model updates.
        """

    def extend_volume(self, volume, new_size):
        """Extend the size of a volume.

        :param volume: The volume to extend.
        :param new_size: The new desired size of the volume.

        Note that if the volume backend doesn't support extending an in-use
        volume, the driver should report online_extend_support=False.
        """

    def create_snapshot(self, snapshot):
        """Creates a snapshot.

        :param snapshot: Information for the snapshot to be created.
        """

    def delete_snapshot(self, snapshot):
        """Deletes a snapshot.

        :param snapshot: The snapshot to delete.
        """

    def create_volume_from_snapshot(self, volume, snapshot):
        """Creates a volume from a snapshot.

        If volume_type extra specs includes 'replication: <is> True'
        the driver needs to create a volume replica (secondary),
        and setup replication between the newly created volume and
        the secondary volume.

        An optional larger size for the new volume can be specified. Drivers
        should check this value and create or expand the new volume to match.

        :param volume: The volume to be created.
        :param snapshot: The snapshot from which to create the volume.
        :returns: A dict of database updates for the new volume.
        """



# Copyright (c) 2015 FUJITSU LIMITED
# Copyright (c) 2012 EMC Corporation.
# Copyright (c) 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

"""
iSCSI Cinder Volume driver for Fujitsu ETERNUS DX S3 series.
"""
import six

from cinder import interface
from cinder.volume import driver
from cinder.volume.drivers.fujitsu.eternus_dx import eternus_dx_common
from oslo_log import log as logging

LOG = logging.getLogger(__name__)


@interface.volumedriver
class FJDXISCSIDriver(driver.ISCSIDriver):
    """iSCSI Cinder Volume Driver for Fujitsu ETERNUS DX S3 series."""

    # ThirdPartySystems wiki page
    CI_WIKI_NAME = "Fujitsu_ETERNUS_CI"
    VERSION = eternus_dx_common.FJDXCommon.VERSION

    def __init__(self, *args, **kwargs):

        super(FJDXISCSIDriver, self).__init__(*args, **kwargs)
        self.common = eternus_dx_common.FJDXCommon(
            'iSCSI',
            configuration=self.configuration)
        self.VERSION = self.common.VERSION

    @staticmethod
    def get_driver_options():
        return eternus_dx_common.FJDXCommon.get_driver_options()

    def check_for_setup_error(self):
        if not self.common.pywbemAvailable:
            LOG.error('pywbem could not be imported! '
                      'pywbem is necessary for this volume driver.')

        return

    def create_volume(self, volume):
        """Create volume."""
        LOG.info('create_volume, volume id: %s, Enter method.', volume['id'])

        element_path, metadata = self.common.create_volume(volume)

        v_metadata = volume.get('volume_metadata')
        if v_metadata:
            for data in v_metadata:
                metadata[data['key']] = data['value']
        else:
            v_metadata = volume.get('metadata', {})
            metadata.update(v_metadata)

        LOG.info('create_volume, info: %s, Exit method.', metadata)
        return {'provider_location': six.text_type(element_path),
                'metadata': metadata}

    def create_volume_from_snapshot(self, volume, snapshot):
        """Creates a volume from a snapshot."""
        LOG.info('create_volume_from_snapshot, '
                 'volume id: %(vid)s, snap id: %(sid)s, Enter method.',
                 {'vid': volume['id'], 'sid': snapshot['id']})

        element_path, metadata = (
            self.common.create_volume_from_snapshot(volume, snapshot))

        v_metadata = volume.get('volume_metadata')
        if v_metadata:
            for data in v_metadata:
                metadata[data['key']] = data['value']
        else:
            v_metadata = volume.get('metadata', {})
            metadata.update(v_metadata)

        LOG.info('create_volume_from_snapshot, '
                 'info: %s, Exit method.', metadata)
        return {'provider_location': six.text_type(element_path),
                'metadata': metadata}

    def create_cloned_volume(self, volume, src_vref):
        """Create cloned volume."""
        LOG.info('create_cloned_volume, '
                 'target volume id: %(tid)s, '
                 'source volume id: %(sid)s, Enter method.',
                 {'tid': volume['id'], 'sid': src_vref['id']})

        element_path, metadata = (
            self.common.create_cloned_volume(volume, src_vref))

        v_metadata = volume.get('volume_metadata')
        if v_metadata:
            for data in v_metadata:
                metadata[data['key']] = data['value']
        else:
            v_metadata = volume.get('metadata', {})
            metadata.update(v_metadata)

        LOG.info('create_cloned_volume, info: %s, Exit method.', metadata)
        return {'provider_location': six.text_type(element_path),
                'metadata': metadata}

    def delete_volume(self, volume):
        """Delete volume on ETERNUS."""
        LOG.info('delete_volume, volume id: %s, Enter method.', volume['id'])

        vol_exist = self.common.delete_volume(volume)

        LOG.info('delete_volume, delete: %s, Exit method.', vol_exist)
        return

    def create_snapshot(self, snapshot):
        """Creates a snapshot."""
        LOG.info('create_snapshot, snap id: %(sid)s, volume id: %(vid)s, '
                 'Enter method.',
                 {'sid': snapshot['id'], 'vid': snapshot['volume_id']})

        element_path, metadata = self.common.create_snapshot(snapshot)

        LOG.info('create_snapshot, info: %s, Exit method.', metadata)
        return {'provider_location': six.text_type(element_path)}

    def delete_snapshot(self, snapshot):
        """Deletes a snapshot."""
        LOG.info('delete_snapshot, snap id: %(sid)s, volume id: %(vid)s, '
                 'Enter method.',
                 {'sid': snapshot['id'], 'vid': snapshot['volume_id']})

        vol_exist = self.common.delete_snapshot(snapshot)

        LOG.info('delete_snapshot, delete: %s, Exit method.', vol_exist)
        return

    def ensure_export(self, context, volume):
        """Driver entry point to get the export info for an existing volume."""
        return

    def create_export(self, context, volume, connector):
        """Driver entry point to get the export info for a new volume."""
        return

    def remove_export(self, context, volume):
        """Driver entry point to remove an export for a volume."""
        return

    def initialize_connection(self, volume, connector):
        """Allow connection to connector and return connection info."""
        LOG.info('initialize_connection, volume id: %(vid)s, '
                 'initiator: %(initiator)s, Enter method.',
                 {'vid': volume['id'], 'initiator': connector['initiator']})

        info = self.common.initialize_connection(volume, connector)

        LOG.info('initialize_connection, info: %s, Exit method.', info)
        return info

    def terminate_connection(self, volume, connector, **kwargs):
        """Disallow connection from connector."""
        initiator = connector.get('initiator') if connector else None

        LOG.info('terminate_connection, volume id: %(vid)s, '
                 'initiator: %(initiator)s, Enter method.',
                 {'vid': volume['id'], 'initiator': initiator})

        map_exist = self.common.terminate_connection(volume, connector)

        LOG.info('terminate_connection, unmap: %s, Exit method.', map_exist)
        return

    def get_volume_stats(self, refresh=False):
        """Get volume stats."""
        LOG.debug('get_volume_stats, refresh: %s, Enter method.', refresh)

        pool_name = None
        if refresh is True:
            data, pool_name = self.common.update_volume_stats()
            backend_name = self.configuration.safe_get('volume_backend_name')
            data['volume_backend_name'] = backend_name or 'FJDXISCSIDriver'
            data['storage_protocol'] = 'iSCSI'
            self._stats = data

        LOG.debug('get_volume_stats, '
                  'pool name: %s, Exit method.', pool_name)
        return self._stats

    def extend_volume(self, volume, new_size):
        """Extend volume."""
        LOG.info('extend_volume, volume id: %s, Enter method.', volume['id'])

        used_pool_name = self.common.extend_volume(volume, new_size)

        LOG.info('extend_volume, used pool name: %s, Exit method.',
                 used_pool_name)



#    Copyright 2016 EMC Corporation
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock
import six

from cinder import objects
from cinder.tests.unit import fake_constants as fake
from cinder.tests.unit import fake_group
from cinder.tests.unit import objects as test_objects


class TestGroupType(test_objects.BaseObjectsTestCase):

    @mock.patch('cinder.db.sqlalchemy.api._group_type_get_full')
    def test_get_by_id(self, group_type_get):
        db_group_type = fake_group.fake_db_group_type()
        group_type_get.return_value = db_group_type
        group_type = objects.GroupType.get_by_id(self.context,
                                                 fake.GROUP_TYPE_ID)
        self._compare(self, db_group_type, group_type)

    @mock.patch('cinder.volume.group_types.create')
    def test_create(self, group_type_create):
        db_group_type = fake_group.fake_db_group_type()
        group_type_create.return_value = db_group_type

        group_type = objects.GroupType(context=self.context)
        group_type.name = db_group_type['name']
        group_type.group_specs = db_group_type['group_specs']
        group_type.is_public = db_group_type['is_public']
        group_type.projects = db_group_type['projects']
        group_type.description = db_group_type['description']
        group_type.create()

        group_type_create.assert_called_once_with(
            self.context, db_group_type['name'],
            db_group_type['group_specs'], db_group_type['is_public'],
            db_group_type['projects'], db_group_type['description'])

    @mock.patch('cinder.volume.group_types.update')
    def test_save(self, group_type_update):
        db_group_type = fake_group.fake_db_group_type()
        group_type = objects.GroupType._from_db_object(self.context,
                                                       objects.GroupType(),
                                                       db_group_type)
        group_type.description = 'foobar'
        group_type.save()
        group_type_update.assert_called_once_with(self.context,
                                                  group_type.id,
                                                  group_type.name,
                                                  group_type.description)

    @mock.patch('cinder.volume.group_types.destroy')
    def test_destroy(self, group_type_destroy):
        db_group_type = fake_group.fake_db_group_type()
        group_type = objects.GroupType._from_db_object(self.context,
                                                       objects.GroupType(),
                                                       db_group_type)
        group_type.destroy()
        self.assertTrue(group_type_destroy.called)
        admin_context = group_type_destroy.call_args[0][0]
        self.assertTrue(admin_context.is_admin)

    @mock.patch('cinder.db.sqlalchemy.api._group_type_get_full')
    def test_refresh(self, group_type_get):
        db_type1 = fake_group.fake_db_group_type()
        db_type2 = db_type1.copy()
        db_type2['description'] = 'foobar'

        # updated description
        group_type_get.side_effect = [db_type1, db_type2]
        group_type = objects.GroupType.get_by_id(self.context,
                                                 fake.GROUP_TYPE_ID)
        self._compare(self, db_type1, group_type)

        # description was updated, so a group type refresh should have a new
        # value for that field
        group_type.refresh()
        self._compare(self, db_type2, group_type)
        if six.PY3:
            call_bool = mock.call.__bool__()
        else:
            call_bool = mock.call.__nonzero__()
        group_type_get.assert_has_calls([mock.call(self.context,
                                                   fake.GROUP_TYPE_ID),
                                         call_bool,
                                         mock.call(self.context,
                                                   fake.GROUP_TYPE_ID)])


class TestGroupTypeList(test_objects.BaseObjectsTestCase):
    @mock.patch('cinder.volume.group_types.get_all_group_types')
    def test_get_all(self, get_all_types):
        db_group_type = fake_group.fake_db_group_type()
        get_all_types.return_value = {db_group_type['name']: db_group_type}

        group_types = objects.GroupTypeList.get_all(self.context)
        self.assertEqual(1, len(group_types))
        TestGroupType._compare(self, db_group_type, group_types[0])

    @mock.patch('cinder.volume.group_types.get_all_group_types')
    def test_get_all_with_pagination(self, get_all_types):
        db_group_type = fake_group.fake_db_group_type()
        get_all_types.return_value = {db_group_type['name']: db_group_type}

        group_types = objects.GroupTypeList.get_all(self.context,
                                                    filters={'is_public':
                                                             True},
                                                    marker=None,
                                                    limit=1,
                                                    sort_keys='id',
                                                    sort_dirs='desc',
                                                    offset=None)
        self.assertEqual(1, len(group_types))
        TestGroupType._compare(self, db_group_type, group_types[0])



#!/usr/bin/env python
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import collections
import os
import subprocess
import textwrap

from cinder.volume import configuration
from cinder.compute import nova

OrderedDict = collections.OrderedDict

BASEDIR = os.path.split(os.path.realpath(__file__))[0] + "/../../"


if __name__ == "__main__":
    os.chdir(BASEDIR)
    opt_file = open("cinder/opts.py", 'w')
    opt_dict = OrderedDict()
    dir_trees_list = []
    REGISTER_OPTS_STR = "CONF.register_opts("
    REGISTER_OPT_STR = "CONF.register_opt("

    license_str = textwrap.dedent(
        """
        # Licensed under the Apache License, Version 2.0 (the "License");
        # you may not use this file except in compliance with the License.
        # You may obtain a copy of the License at
        #
        #    http://www.apache.org/licenses/LICENSE-2.0
        #
        # Unless required by applicable law or agreed to in writing, software
        # distributed under the License is distributed on an "AS IS" BASIS,
        # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
        # implied.
        # See the License for the specific language governing permissions and
        # limitations under the License.\n
    """)
    opt_file.write(license_str)

    edit_header = textwrap.dedent(
        """
        ###################################################################
        # WARNING!
        #
        # Do not edit this file directly. This file should be generated by
        # running the command "tox -e genopts" any time a config option
        # has been added, changed, or removed.
        ###################################################################\n
    """)
    opt_file.write(edit_header)

    opt_file.write("import itertools\n\n")

    opt_file.write("from keystoneauth1 import loading\n\n")
    # NOTE(geguileo): We need to register all OVOs before importing any other
    # cinder files, otherwise any decorator that uses cinder.objects.YYY will
    # fail with exception AttributeError: 'module' object has no attribute
    # 'YYY' when running tox -egenconfig
    opt_file.write("from cinder import objects\nobjects.register_all()\n\n")

    targetdir = 'cinder'

    common_string = ('find ' + targetdir + ' -type f -name "*.py" !  '
                     '-path "*/tests/*" -exec grep -l "%s" {} '
                     '+  | sed -e "s|^' + BASEDIR +
                     '|/|g" | sort -u')

    cmd_opts = common_string % REGISTER_OPTS_STR
    output_opts = subprocess.check_output(  # nosec : command is hardcoded
        '{}'.format(cmd_opts), shell=True,
        universal_newlines=True)
    dir_trees_list = output_opts.split()

    cmd_opt = common_string % REGISTER_OPT_STR
    output_opt = subprocess.check_output(  # nosec : command is hardcoded
        '{}'.format(cmd_opt), shell=True,
        universal_newlines=True)
    temp_list = output_opt.split()

    for item in temp_list:
        dir_trees_list.append(item)
    dir_trees_list.sort()

    flag = False

    def _check_import(aline):
        if len(aline) > 79:
            new_lines = aline.partition(' as ')
            return new_lines
        else:
            return [aline]

    for atree in dir_trees_list:

        if atree in ["tools/config/generate_cinder_opts.py",
                     "cinder/tests/hacking/checks.py",
                     "cinder/volume/configuration.py",
                     "cinder/test.py",
                     "cinder/cmd/status.py"]:
            continue

        dirs_list = atree.split('/')

        import_module = "from "
        init_import_module = ""
        import_name = ""

        for dir in dirs_list:
            if dir.find(".py") == -1:
                import_module += dir + "."
                init_import_module += dir + "."
                import_name += dir + "_"
            else:
                if dir[:-3] != "__init__":
                    import_name += dir[:-3].replace("_", "")
                    import_module = (import_module[:-1] + " import " +
                                     dir[:-3] + " as " + import_name)
                    lines = _check_import(import_module)
                    if len(lines) > 1:
                        opt_file.write(lines[0] + lines[1] + "\\\n")
                        opt_file.write("    " + lines[2] + "\n")
                    else:
                        opt_file.write(lines[0] + "\n")

                else:
                    import_name = import_name[:-1].replace('/', '.')
                    init_import = atree[:-12].replace('/', '.')
                    opt_file.write("import " + init_import + "\n")

                    flag = True
        if flag is False:
            opt_dict[import_name] = atree
        else:
            opt_dict[init_import_module.strip(".")] = atree

        flag = False

    registered_opts_dict = OrderedDict([('DEFAULT', [])])

    def _write_item(opts):
        list_name = opts[-3:]
        if list_name.lower() == "opt":
            line_to_write = "                [" + opts.strip("\n") + "],\n"
            opt_line = _check_line_length(line_to_write)
            if len(opt_line) > 1:
                opt_file.write(opt_line[0] + opt_line[1] + "\n")
                opt_file.write("                    " + opt_line[2])
            else:
                opt_file.write(opt_line[0])
        else:
            line_to_write = "                " + opts.strip("\n") + ",\n"
            opt_line = _check_line_length(line_to_write)
            if len(opt_line) > 1:
                opt_file.write(opt_line[0] + opt_line[1] + "\n")
                opt_file.write("                " + opt_line[2])
            else:
                opt_file.write(opt_line[0])
        if opts.endswith('service_user_opts'):
            su_dnt = " " * 16
            su_plg = su_dnt + "loading.get_auth_plugin_conf_options"
            opt_file.write(
                su_plg + "('v3password'),\n"
                + su_dnt + "loading.get_session_conf_options(),\n")

    def _retrieve_name(aline):
        if REGISTER_OPT_STR in aline:
            str_to_replace = REGISTER_OPT_STR
        else:
            str_to_replace = REGISTER_OPTS_STR
        return aline.replace(str_to_replace, "")

    def _check_line_length(aline):
        if len(aline) > 79:
            temp = aline.split(".")
            lines_to_write = []

            for section in temp:
                lines_to_write.append(section)
                lines_to_write.append('.')

            return lines_to_write

        else:
            return [aline]

    for key in opt_dict:
        fd = os.open(opt_dict[key], os.O_RDONLY)
        afile = os.fdopen(fd, "r")

        for aline in afile:
            exists = aline.find("CONF.register_opt")
            if exists != -1:
                # TODO(kjnelson) FIX THIS LATER. These are instances where
                # CONF.register_opts is happening without actually registering
                # real lists of opts

                exists = aline.find('base_san_opts')
                if (exists != -1) or (key == 'cinder_volume_configuration'):
                    continue

                group_exists = aline.find(', group=')
                formatted_opt = _retrieve_name(aline[: group_exists])
                formatted_opt = formatted_opt.replace(')', '').strip()
                if group_exists != -1:
                    group_name = aline[group_exists:-1].replace(
                        ', group=\"\'', '').replace(
                        ', group=', '').strip(
                        "\'\")").upper()

                    # NOTE(dulek): Hack to resolve constants manually.
                    if (group_name.endswith('SHARED_CONF_GROUP')
                            or group_name.lower() == 'backend_defaults'):
                        group_name = configuration.SHARED_CONF_GROUP
                    if (group_name == 'NOVA_GROUP'):
                        group_name = nova.NOVA_GROUP

                    if group_name in registered_opts_dict:
                        line = key + "." + formatted_opt
                        registered_opts_dict[group_name].append(line)
                    else:
                        line = key + "." + formatted_opt
                        registered_opts_dict[group_name] = [line]
                else:
                    line = key + "." + formatted_opt
                    registered_opts_dict['DEFAULT'].append(line)

    setup_str = ("\n\n"
                 "def list_opts():\n"
                 "    return [\n")
    opt_file.write(setup_str)

    registered_opts_dict = OrderedDict(sorted(registered_opts_dict.items(),
                                              key = lambda x: x[0]))

    for key in registered_opts_dict:
        # NOTE(jsbryant): We need to have 'DEFAULT' in uppercase but any
        # other section using uppercase causes a Sphinx warning.
        if (key == 'DEFAULT'):
            section_start_str = ("        ('" + key + "',\n"
                                 "            itertools.chain(\n")
        else:
           section_start_str = ("        ('" + key.lower() + "',\n"
                                "            itertools.chain(\n")
        opt_file.write(section_start_str)
        for item in registered_opts_dict[key]:
            _write_item(item)
        section_end_str = "            )),\n"
        opt_file.write(section_end_str)

    closing_str = ("    ]\n")
    opt_file.write(closing_str)
    opt_file.close()



# Copyright (c) 2015 Infortrend Technology, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from cinder import test
from cinder.volume.drivers.infortrend.raidcmd_cli import cli_factory as cli


class InfortrendCLITestData(object):

    """CLI Test Data."""

    # Infortrend entry
    fake_lv_id = ['5DE94FF775D81C30', '1234567890', 'HK3345678']

    fake_partition_id = ['6A41315B0EDC8EB7', '51B4283E4E159173',
                         '987654321', '123456789',
                         '2667FE351FC505AE', '53F3E98141A2E871']

    fake_pair_id = ['55D790F8350B036B', '095A184B0ED2DB10']

    fake_snapshot_id = ['2C7A8D211F3B1E36', '60135EE53C14D5EB',
                        '4884610D11FD3335', '5C44BE0A776A2804']

    fake_snapshot_name = ['9e8b27e9-568c-44ca-bd7c-2c7af96ab248',
                          '35e8ba6e-3372-4e67-8464-2b68758f3aeb',
                          'f69696ea-26fc-4f4c-97335-e3ce33ee563',
                          'cinder-unmanaged-f31d8326-c2d8-4668-']

    fake_data_port_ip = ['172.27.0.1', '172.27.0.2',
                         '172.27.0.3', '172.27.0.4',
                         '172.27.0.5', '172.27.0.6']

    fake_model = ['DS S12F-G2852-6']

    fake_manage_port_ip = ['172.27.0.10']

    fake_system_id = ['DEEC']

    fake_host_ip = ['172.27.0.2']

    fake_target_wwnns = ['100123D02300DEEC', '100123D02310DEEC']

    fake_target_wwpns = ['110123D02300DEEC', '120123D02300DEEC',
                         '110123D02310DEEC', '120123D02310DEEC']

    fake_initiator_wwnns = ['2234567890123456', '2234567890543216']

    fake_initiator_wwpns = ['1234567890123456', '1234567890543216']

    fake_initiator_iqn = ['iqn.1991-05.com.infortrend:pc123',
                          'iqn.1991-05.com.infortrend:pc456']

    fake_lun_map = [0, 1, 2]

    # cinder entry
    test_provider_location = [(
        'system_id^%s@partition_id^%s') % (
            int(fake_system_id[0], 16), fake_partition_id[0]), (
        'system_id^%s@partition_id^%s') % (
            int(fake_system_id[0], 16), fake_partition_id[1])
    ]

    test_volume = {
        'id': '5aa119a8-d25b-45a7-8d1b-88e127885635',
        'size': 1,
        'name': 'Part-1',
        'host': 'infortrend-server1@backend_1#LV-1',
        'name_id': '5aa119a8-d25b-45a7-8d1b-88e127885635',
        'provider_auth': None,
        'project_id': 'project',
        'display_name': None,
        'display_description': 'Part-1',
        'volume_type_id': None,
        'status': 'available',
        'provider_location': test_provider_location[0],
        'volume_attachment': [],
    }

    test_volume_1 = {
        'id': '5aa119a8-d25b-45a7-8d1b-88e127885634',
        'size': 1,
        'name': 'Part-1',
        'host': 'infortrend-server1@backend_1#LV-1',
        'name_id': '5aa119a8-d25b-45a7-8d1b-88e127885635',
        'provider_auth': None,
        'project_id': 'project',
        'display_name': None,
        'display_description': 'Part-1',
        'volume_type_id': None,
        'status': 'in-use',
        'provider_location': test_provider_location[1],
        'volume_attachment': [],
    }

    test_dst_volume = {
        'id': '6bb119a8-d25b-45a7-8d1b-88e127885666',
        'size': 1,
        'name': 'Part-1-Copy',
        'host': 'infortrend-server1@backend_1#LV-1',
        'name_id': '6bb119a8-d25b-45a7-8d1b-88e127885666',
        'provider_auth': None,
        'project_id': 'project',
        'display_name': None,
        '_name_id': '6bb119a8-d25b-45a7-8d1b-88e127885666',
        'display_description': 'Part-1-Copy',
        'volume_type_id': None,
        'provider_location': '',
        'volume_attachment': [],
    }

    test_ref_volume = {
        'source-id': fake_partition_id[0],
        'size': 1,
    }

    test_ref_volume_with_id = {
        'source-id': '6bb119a8-d25b-45a7-8d1b-88e127885666',
        'size': 1,
    }

    test_ref_volume_with_name = {
        'source-name': 'import_into_openstack',
        'size': 1,
    }

    test_snapshot = {
        'id': 'ffa9bc5e-1172-4021-acaf-cdcd78a9584d',
        'volume_id': test_volume['id'],
        'volume_name': test_volume['name'],
        'volume_size': 2,
        'project_id': 'project',
        'display_name': None,
        'display_description': 'SI-1',
        'volume_type_id': None,
        'provider_location': fake_snapshot_id[0],
    }
    test_snapshot_without_provider_location = {
        'id': 'ffa9bc5e-1172-4021-acaf-cdcd78a9584d',
        'volume_id': test_volume['id'],
        'volume_name': test_volume['name'],
        'volume_size': 2,
        'project_id': 'project',
        'display_name': None,
        'display_description': 'SI-1',
        'volume_type_id': None,
    }

    test_iqn = [(
        'iqn.2002-10.com.infortrend:raid.uid%s.%s%s%s') % (
            int(fake_system_id[0], 16), 1, 0, 1), (
        'iqn.2002-10.com.infortrend:raid.uid%s.%s%s%s') % (
            int(fake_system_id[0], 16), 1, 0, 1), (
        'iqn.2002-10.com.infortrend:raid.uid%s.%s%s%s') % (
            int(fake_system_id[0], 16), 2, 0, 1),
    ]

    test_iscsi_properties = {
        'driver_volume_type': 'iscsi',
        'data': {
            'target_discovered': True,
            'target_portal': '%s:3260' % fake_data_port_ip[2],
            'target_iqn': test_iqn[0],
            'target_lun': fake_lun_map[0],
            'volume_id': test_volume['id'],
        },
    }

    test_iscsi_properties_with_mcs = {
        'driver_volume_type': 'iscsi',
        'data': {
            'target_discovered': True,
            'target_portal': '%s:3260' % fake_data_port_ip[4],
            'target_iqn': test_iqn[2],
            'target_lun': fake_lun_map[0],
            'volume_id': test_volume['id'],
        },
    }

    test_iscsi_properties_with_mcs_1 = {
        'driver_volume_type': 'iscsi',
        'data': {
            'target_discovered': True,
            'target_portal': '%s:3260' % fake_data_port_ip[4],
            'target_iqn': test_iqn[2],
            'target_lun': fake_lun_map[1],
            'volume_id': test_volume_1['id'],
        },
    }

    test_iqn_empty_map = [(
        'iqn.2002-10.com.infortrend:raid.uid%s.%s%s%s') % (
            int(fake_system_id[0], 16), 0, 0, 1),
    ]

    test_iscsi_properties_empty_map = {
        'driver_volume_type': 'iscsi',
        'data': {
            'target_discovered': True,
            'target_portal': '%s:3260' % fake_data_port_ip[0],
            'target_iqn': test_iqn_empty_map[0],
            'target_lun': fake_lun_map[0],
            'volume_id': test_volume['id'],
        },
    }

    test_initiator_target_map = {
        fake_initiator_wwpns[0]: fake_target_wwpns[0:2],
        fake_initiator_wwpns[1]: fake_target_wwpns[0:2],
    }

    test_fc_properties = {
        'driver_volume_type': 'fibre_channel',
        'data': {
            'target_discovered': True,
            'target_lun': fake_lun_map[0],
            'target_wwn': fake_target_wwpns[0:2],
            'initiator_target_map': test_initiator_target_map,
        },
    }

    test_initiator_target_map_specific_channel = {
        fake_initiator_wwpns[0]: [fake_target_wwpns[1]],
        fake_initiator_wwpns[1]: [fake_target_wwpns[1]],
    }

    test_fc_properties_with_specific_channel = {
        'driver_volume_type': 'fibre_channel',
        'data': {
            'target_discovered': True,
            'target_lun': fake_lun_map[0],
            'target_wwn': [fake_target_wwpns[1]],
            'initiator_target_map': test_initiator_target_map_specific_channel,
        },
    }

    test_target_wwpns_map_multipath_r_model = [
        fake_target_wwpns[0],
        fake_target_wwpns[2],
        fake_target_wwpns[1],
        fake_target_wwpns[3],
    ]

    test_initiator_target_map_multipath_r_model = {
        fake_initiator_wwpns[0]: test_target_wwpns_map_multipath_r_model[:],
        fake_initiator_wwpns[1]: test_target_wwpns_map_multipath_r_model[:],
    }

    test_fc_properties_multipath_r_model = {
        'driver_volume_type': 'fibre_channel',
        'data': {
            'target_discovered': True,
            'target_lun': fake_lun_map[0],
            'target_wwn': test_target_wwpns_map_multipath_r_model[:],
            'initiator_target_map':
                test_initiator_target_map_multipath_r_model,
        },
    }

    test_initiator_target_map_zoning = {
        fake_initiator_wwpns[0].lower():
            [x.lower() for x in fake_target_wwpns[0:2]],
        fake_initiator_wwpns[1].lower():
            [x.lower() for x in fake_target_wwpns[0:2]],
    }

    test_fc_properties_zoning = {
        'driver_volume_type': 'fibre_channel',
        'data': {
            'target_discovered': True,
            'target_lun': fake_lun_map[0],
            'target_wwn': [x.lower() for x in fake_target_wwpns[0:2]],
            'initiator_target_map': test_initiator_target_map_zoning,
        },
    }

    test_initiator_target_map_zoning_r_model = {
        fake_initiator_wwpns[0].lower():
            [x.lower() for x in fake_target_wwpns[1:3]],
        fake_initiator_wwpns[1].lower():
            [x.lower() for x in fake_target_wwpns[1:3]],
    }

    test_fc_properties_zoning_r_model = {
        'driver_volume_type': 'fibre_channel',
        'data': {
            'target_discovered': True,
            'target_lun': fake_lun_map[0],
            'target_wwn': [x.lower() for x in fake_target_wwpns[1:3]],
            'initiator_target_map': test_initiator_target_map_zoning_r_model,
        },
    }

    test_fc_terminate_conn_info = {
        'driver_volume_type': 'fibre_channel',
        'data': {
            'initiator_target_map': test_initiator_target_map_zoning,
        },
    }

    test_connector_iscsi = {
        'ip': fake_host_ip[0],
        'initiator': fake_initiator_iqn[0],
        'host': 'infortrend-server1@backend_1',
    }

    test_connector_iscsi_1 = {
        'ip': fake_host_ip[0],
        'initiator': fake_initiator_iqn[1],
        'host': 'infortrend-server1@backend_1',
    }

    test_connector_fc = {
        'wwpns': fake_initiator_wwpns,
        'wwnns': fake_initiator_wwnns,
        'host': 'infortrend-server1@backend_1',
    }

    fake_pool = {
        'pool_name': 'LV-2',
        'pool_id': fake_lv_id[1],
        'total_capacity_gb': 1000,
        'free_capacity_gb': 1000,
        'reserved_percentage': 0,
        'QoS_support': False,
        'thin_provisioning_support': False,
    }

    test_pools_full = [{
        'pool_name': 'LV-1',
        'pool_id': fake_lv_id[0],
        'location_info': 'Infortrend:' + fake_system_id[0],
        'total_capacity_gb': round(857982.0 / 1024, 2),
        'free_capacity_gb': round(841978.0 / 1024, 2),
        'reserved_percentage': 0,
        'QoS_support': False,
        'thick_provisioning_support': True,
        'thin_provisioning_support': False,
    }]

    test_volume_states_full = {
        'volume_backend_name': 'infortrend_backend_1',
        'vendor_name': 'Infortrend',
        'driver_version': '99.99',
        'storage_protocol': 'iSCSI',
        'model_type': 'R',
        'status': 'Connected',
        'system_id': fake_system_id[0],
        'pools': test_pools_full,
    }

    test_pools_thin = [{
        'pool_name': 'LV-1',
        'pool_id': fake_lv_id[0],
        'location_info': 'Infortrend:' + fake_system_id[0],
        'total_capacity_gb': round(857982.0 / 1024, 2),
        'free_capacity_gb': round(841978.0 / 1024, 2),
        'reserved_percentage': 0,
        'QoS_support': False,
        'thick_provisioning_support': True,
        'thin_provisioning_support': True,
        'provisioned_capacity_gb':
            round((40000) / 1024, 2),
        'max_over_subscription_ratio': 20.0,
    }]

    test_volume_states_thin = {
        'volume_backend_name': 'infortrend_backend_1',
        'vendor_name': 'Infortrend',
        'driver_version': '99.99',
        'storage_protocol': 'iSCSI',
        'model_type': 'R',
        'status': 'Connected',
        'system_id': fake_system_id[0],
        'pools': test_pools_thin,
    }

    test_host = {
        'host': 'infortrend-server1@backend_1',
        'capabilities': test_volume_states_thin,
    }

    test_migrate_volume_states = {
        'volume_backend_name': 'infortrend_backend_1',
        'vendor_name': 'Infortrend',
        'driver_version': '99.99',
        'storage_protocol': 'iSCSI',
        'pool_name': 'LV-1',
        'pool_id': fake_lv_id[1],
        'location_info': 'Infortrend:' + fake_system_id[0],
        'total_capacity_gb': round(857982.0 / 1024, 2),
        'free_capacity_gb': round(841978.0 / 1024, 2),
        'reserved_percentage': 0,
        'QoS_support': False,
    }

    test_migrate_host = {
        'host': 'infortrend-server1@backend_1#LV-2',
        'capabilities': test_migrate_volume_states,
    }

    test_migrate_volume_states_2 = {
        'volume_backend_name': 'infortrend_backend_1',
        'vendor_name': 'Infortrend',
        'driver_version': '99.99',
        'storage_protocol': 'iSCSI',
        'pool_name': 'LV-1',
        'pool_id': fake_lv_id[1],
        'location_info': 'Infortrend:' + fake_system_id[0],
        'total_capacity_gb': round(857982.0 / 1024, 2),
        'free_capacity_gb': round(841978.0 / 1024, 2),
        'reserved_percentage': 0,
        'QoS_support': False,
    }

    test_migrate_host_2 = {
        'host': 'infortrend-server1@backend_1#LV-1',
        'capabilities': test_migrate_volume_states_2,
    }

    fake_host = {
        'host': 'infortrend-server1@backend_1',
        'capabilities': {},
    }

    fake_volume_id = [test_volume['id'], test_dst_volume['id']]

    fake_lookup_map = {
        '12345678': {
            'initiator_port_wwn_list':
                [x.lower() for x in fake_initiator_wwpns],
            'target_port_wwn_list':
                [x.lower() for x in fake_target_wwpns[0:2]],
        },
    }

    fake_lookup_map_r_model = {
        '12345678': {
            'initiator_port_wwn_list':
                [x.lower() for x in fake_initiator_wwpns[:]],
            'target_port_wwn_list':
                [x.lower() for x in fake_target_wwpns[1:3]],
        },
    }

    test_new_type = {
        'name': 'type0',
        'qos_specs_id': None,
        'deleted': False,
        'extra_specs': {'infortrend:provisioning': 'thin'},
        'id': '28c8f82f-416e-148b-b1ae-2556c032d3c0',
    }

    test_diff = {'extra_specs': {'infortrend:provisioning': ('full', 'thin')}}

    def get_fake_cli_failed(self):
        return """
ift cli command
CLI: No selected device
Return: 0x000c

RAIDCmd:>
"""

    def get_fake_cli_failed_with_network(self):
        return """
ift cli command
CLI: Not exist: There is no such partition: 3345678
Return: 0x000b

RAIDCmd:>
"""

    def get_fake_cli_succeed(self):
        return """
ift cli command
CLI: Successful: 0 mapping(s) shown
Return: 0x0000

RAIDCmd:>
"""

    def get_test_show_empty_list(self):
        return (0, [])

    def get_test_show_snapshot(self, partition_id=None, snapshot_id=None):
        if partition_id and snapshot_id:
            return (0, [{
                'Map': 'No',
                'Partition-ID': partition_id,
                'SI-ID': snapshot_id,
                'Name': '---',
                'Activated-time': 'Thu, Jan 09 01:33:11 2020',
                'Index': '1',
            }])
        else:
            return (0, [{
                'Map': 'No',
                'Partition-ID': self.fake_partition_id[0],
                'SI-ID': self.fake_snapshot_id[0],
                'Name': '---',
                'Activated-time': 'Thu, Jan 09 01:33:11 2020',
                'Index': '1',
            }, {
                'Map': 'No',
                'Partition-ID': self.fake_partition_id[0],
                'SI-ID': self.fake_snapshot_id[1],
                'Name': '---',
                'Activated-time': 'Thu, Jan 09 01:35:50 2020',
                'Index': '2',
            }])

    def get_test_show_snapshot_named(self):
        return (0, [{
            'Map': 'No',
            'Partition-ID': self.fake_partition_id[0],
            'SI-ID': self.fake_snapshot_id[0],
            'Name': self.fake_snapshot_name[0],
            'Activated-time': 'Thu, Jan 09 01:33:11 2020',
            'Index': '1',
        }, {
            'Map': 'No',
            'Partition-ID': self.fake_partition_id[1],
            'SI-ID': self.fake_snapshot_id[1],
            'Name': self.fake_snapshot_name[1],
            'Activated-time': 'Thu, Jan 09 01:35:50 2020',
            'Index': '1',
        }])

    def get_fake_show_snapshot(self):
        msg = r"""
show si
\/\/\/-
\
/
-

\
/
-
\/-\/- Index  SI-ID  Name  Partition-ID  Map  Activated-time
---------------------------------------------------------------------------------
 1      %s     ---   %s            No   Thu, Jan 09 01:33:11 2020
 2      %s     ---   %s            No   Thu, Jan 09 01:35:50 2020

CLI: Successful: 2 snapshot image(s) shown
Return: 0x0000

RAIDCmd:>
"""
        return msg % (self.fake_snapshot_id[0],
                      self.fake_partition_id[0],
                      self.fake_snapshot_id[1],
                      self.fake_partition_id[0])

    def get_test_show_snapshot_detail_filled_block(self):
        return (0, [{
            'Mapped': 'Yes',
            'Created-time': 'Wed, Jun 10 10:57:16 2015',
            'ID': self.fake_snapshot_id[0],
            'Last-modification-time': 'Wed, Jun 10 10:57:16 2015',
            'Description': '---',
            'Total-filled-block': '1',
            'LV-ID': self.fake_lv_id[0],
            'Activation-schedule-time': 'Not Actived',
            'Mapping': 'CH:0/ID:0/LUN:1',
            'Index': '1',
            'Used': '0',
            'Name': '---',
            'Valid-filled-block': '0',
            'Partition-ID': self.fake_partition_id[0],
        }])

    def get_test_show_snapshot_detail(self):
        return (0, [{
            'Mapped': 'Yes',
            'Created-time': 'Wed, Jun 10 10:57:16 2015',
            'ID': self.fake_snapshot_id[0],
            'Last-modification-time': 'Wed, Jun 10 10:57:16 2015',
            'Description': '---',
            'Total-filled-block': '0',
            'LV-ID': self.fake_lv_id[0],
            'Activation-schedule-time': 'Not Actived',
            'Mapping': 'CH:0/ID:0/LUN:1',
            'Index': '1',
            'Used': '0',
            'Name': '---',
            'Valid-filled-block': '0',
            'Partition-ID': self.fake_partition_id[0],
        }])

    def get_test_show_snapshot_get_manage(self):
        """Show 4 si for api `list si`: 1.Mapped 2.Managed 3.Free 4.WrongLV"""

        return (0, [{
            'ID': self.fake_snapshot_id[0],
            'Index': '1',
            'Name': self.fake_snapshot_name[0],
            'Partition-ID': self.fake_partition_id[0],
            'LV-ID': self.fake_lv_id[0],
            'Created-time': 'Fri, Dec 23 07:54:33 2016',
            'Last-modification-time': 'Fri, Dec 23 07:54:33 2016',
            'Activated-time': 'Fri, Dec 23 08:29:41 2016',
            'Activation-schedule-time': 'Not Actived',
            'Used': '0',
            'Valid-filled-block': '0',
            'Total-filled-block': '0',
            'Description': '---',
            'Mapped': 'No',
            'Mapping': '---',
            'Backup-to-Cloud': 'false',
            'Status': 'OK',
            'Progress': '---',
        }, {
            'ID': self.fake_snapshot_id[1],
            'Index': '2',
            'Name': self.fake_snapshot_name[1],
            'Partition-ID': self.fake_partition_id[1],
            'LV-ID': self.fake_lv_id[0],
            'Created-time': 'Fri, Dec 23 07:54:33 2016',
            'Last-modification-time': 'Fri, Dec 23 07:54:33 2016',
            'Activated-time': 'Fri, Dec 23 08:29:41 2016',
            'Activation-schedule-time': 'Not Actived',
            'Used': '0',
            'Valid-filled-block': '0',
            'Total-filled-block': '0',
            'Description': '---',
            'Mapped': 'No',
            'Mapping': '---',
            'Backup-to-Cloud': 'false',
            'Status': 'OK',
            'Progress': '---'
        }, {
            'ID': self.fake_snapshot_id[2],
            'Index': '1',
            'Name': self.fake_snapshot_name[2],
            'Partition-ID': self.fake_partition_id[2],
            'LV-ID': self.fake_lv_id[1],
            'Created-time': 'Fri, Dec 23 07:54:33 2016',
            'Last-modification-time': 'Fri, Dec 23 07:54:33 2016',
            'Activated-time': 'Fri, Dec 23 08:29:41 2016',
            'Activation-schedule-time': 'Not Actived',
            'Used': '0',
            'Valid-filled-block': '0',
            'Total-filled-block': '0',
            'Description': '---',
            'Mapped': 'No',
            'Mapping': '---',
            'Backup-to-Cloud': 'false',
            'Status': 'OK',
            'Progress': '---',
        }, {
            'ID': self.fake_snapshot_id[3],
            'Index': '1',
            'Name': 'test-get-snapshot-list',
            # Part ID from get_test_show_partition_detail()
            'Partition-ID': '123123123123',
            'LV-ID': '987654321',
            'Created-time': 'Fri, Dec 23 07:54:33 2016',
            'Last-modification-time': 'Fri, Dec 23 07:54:33 2016',
            'Activated-time': 'Fri, Dec 23 08:29:41 2016',
            'Activation-schedule-time': 'Not Actived',
            'Used': '0',
            'Valid-filled-block': '0',
            'Total-filled-block': '0',
            'Description': '---',
            'Mapped': 'No',
            'Mapping': '---',
            'Backup-to-Cloud': 'false',
            'Status': 'OK',
            'Progress': '---'
        }])

    def get_fake_show_snapshot_detail(self):
        msg = """
show si -l
 ID: %s
 Index: 1
 Name: ---
 Partition-ID: %s
 LV-ID: %s
 Created-time: Wed, Jun 10 10:57:16 2015
 Last-modification-time: Wed, Jun 10 10:57:16 2015
 Activation-schedule-time: Not Actived
 Used: 0
 Valid-filled-block: 0
 Total-filled-block: 0
 Description: ---
 Mapped: Yes
 Mapping: CH:0/ID:0/LUN:1

CLI: Successful: 1 snapshot image(s) shown
Return: 0x0000

RAIDCmd:>
"""
        return msg % (self.fake_snapshot_id[0],
                      self.fake_partition_id[0],
                      self.fake_lv_id[0])

    def get_test_show_net(self):
        return (0, [{
            'Slot': 'slotA',
            'MAC': '10D02380DEEC',
            'ID': '1',
            'IPv4': self.fake_data_port_ip[0],
            'Mode': 'Disabled',
            'IPv6': '---',
        }, {
            'Slot': 'slotB',
            'MAC': '10D02390DEEC',
            'ID': '1',
            'IPv4': self.fake_data_port_ip[1],
            'Mode': 'Disabled',
            'IPv6': '---',
        }, {
            'Slot': 'slotA',
            'MAC': '10D02340DEEC',
            'ID': '2',
            'IPv4': self.fake_data_port_ip[2],
            'Mode': 'Disabled',
            'IPv6': '---',
        }, {
            'Slot': 'slotB',
            'MAC': '10D02350DEEC',
            'ID': '2',
            'IPv4': self.fake_data_port_ip[3],
            'Mode': 'Disabled',
            'IPv6': '---',
        }, {
            'Slot': 'slotA',
            'MAC': '10D02310DEEC',
            'ID': '4',
            'IPv4': self.fake_data_port_ip[4],
            'Mode': 'Disabled',
            'IPv6': '---',
        }, {
            'Slot': 'slotB',
            'MAC': '10D02320DEEC',
            'ID': '4',
            'IPv4': self.fake_data_port_ip[5],
            'Mode': 'Disabled',
            'IPv6': '---',
        }, {
            'Slot': '---',
            'MAC': '10D023077124',
            'ID': '32',
            'IPv4': '172.27.1.1',
            'Mode': 'Disabled',
            'IPv6': '---',
        }])

    def get_fake_show_net(self):
        msg = """
show net
 ID  MAC           Mode  IPv4            Mode      IPv6  Slot
---------------------------------------------------------------
 1   10D02380DEEC  DHCP  %s              Disabled  ---   slotA
 1   10D02390DEEC  DHCP  %s              Disabled  ---   slotB
 2   10D02340DEEC  DHCP  %s              Disabled  ---   slotA
 2   10D02350DEEC  DHCP  %s              Disabled  ---   slotB
 4   10D02310DEEC  DHCP  %s              Disabled  ---   slotA
 4   10D02320DEEC  DHCP  %s              Disabled  ---   slotB
 32  10D023077124  DHCP  172.27.1.1      Disabled  ---   ---

CLI: Successful: 2 record(s) found
Return: 0x0000

RAIDCmd:>
"""
        return msg % (self.fake_data_port_ip[0], self.fake_data_port_ip[1],
                      self.fake_data_port_ip[2], self.fake_data_port_ip[3],
                      self.fake_data_port_ip[4], self.fake_data_port_ip[5])

    def get_test_show_net_detail(self):
        return (0, [{
            'Slot': 'slotA',
            'IPv4-mode': 'DHCP',
            'ID': '1',
            'IPv6-address': '---',
            'Net-mask': '---',
            'IPv4-address': '---',
            'Route': '---',
            'Gateway': '---',
            'IPv6-mode': 'Disabled',
            'MAC': '00D023877124',
            'Prefix-length': '---',
        }, {
            'Slot': '---',
            'IPv4-mode': 'DHCP',
            'ID': '32',
            'IPv6-address': '---',
            'Net-mask': '255.255.240.0',
            'IPv4-address': '172.27.112.245',
            'Route': '---',
            'Gateway': '172.27.127.254',
            'IPv6-mode': 'Disabled',
            'MAC': '00D023077124',
            'Prefix-length': '---',
        }])

    def get_fake_show_net_detail(self):
        msg = """
show net -l
 ID: 1
 MAC: 00D023877124
 IPv4-mode: DHCP
 IPv4-address: ---
 Net-mask: ---
 Gateway: ---
 IPv6-mode: Disabled
 IPv6-address: ---
 Prefix-length: ---
 Route: ---
 Slot: slotA

 ID: 32
 MAC: 00D023077124
 IPv4-mode: DHCP
 IPv4-address: 172.27.112.245
 Net-mask: 255.255.240.0
 Gateway: 172.27.127.254
 IPv6-mode: Disabled
 IPv6-address: ---
 Prefix-length: ---
 Route: ---
 Slot: ---

CLI: Successful: 3 record(s) found
Return: 0x0000

RAIDCmd:>
"""
        return msg

    def get_test_show_partition(self, volume_id=None, pool_id=None):
        result = [{
            'ID': self.fake_partition_id[0],
            'Used': '20000',
            'Name': self.fake_volume_id[0],
            'Size': '20000',
            'Min-reserve': '20000',
            'LV-ID': self.fake_lv_id[0],
        }, {
            'ID': self.fake_partition_id[1],
            'Used': '20000',
            'Name': self.fake_volume_id[1],
            'Size': '20000',
            'Min-reserve': '20000',
            'LV-ID': self.fake_lv_id[0],
        }]
        if volume_id and pool_id:
            result.append({
                'ID': self.fake_partition_id[2],
                'Used': '20000',
                'Name': volume_id,
                'Size': '20000',
                'Min-reserve': '20000',
                'LV-ID': pool_id,
            })
        return (0, result)

    def get_fake_show_partition(self):
        msg = """
show part
 ID  Name         LV-ID  Size     Used     Min-reserve
---------------------------------------------------
 %s  %s           %s     20000    20000    20000
 %s  %s           %s     20000    20000    20000

CLI: Successful: 3 partition(s) shown
Return: 0x0000

RAIDCmd:>
"""
        return msg % (self.fake_partition_id[0],
                      self.fake_volume_id[0],
                      self.fake_lv_id[0],
                      self.fake_partition_id[1],
                      self.fake_volume_id[1],
                      self.fake_lv_id[0])

    def get_test_show_partition_detail_for_map(
            self, partition_id, mapped='true'):
        result = [{
            'LV-ID': self.fake_lv_id[0],
            'Mapping': 'CH:1/ID:0/LUN:0, CH:1/ID:0/LUN:1',
            'Used': '20000',
            'Size': '20000',
            'ID': partition_id,
            'Progress': '---',
            'Min-reserve': '20000',
            'Last-modification-time': 'Wed, Jan 08 20:23:23 2020',
            'Valid-filled-block': '100',
            'Name': self.fake_volume_id[0],
            'Mapped': mapped,
            'Total-filled-block': '100',
            'Creation-time': 'Wed, Jan 08 20:23:23 2020',
        }]
        return (0, result)

    def get_test_show_partition_detail(self, volume_id=None, pool_id=None):
        result = [{
            'LV-ID': self.fake_lv_id[0],
            'Mapping': 'CH:1/ID:0/LUN:0, CH:1/ID:0/LUN:1, CH:4/ID:0/LUN:0',
            'Used': '20000',
            'Size': '20000',
            'ID': self.fake_partition_id[0],
            'Progress': '---',
            'Min-reserve': '20000',
            'Last-modification-time': 'Wed, Jan 08 20:23:23 2020',
            'Valid-filled-block': '100',
            'Name': self.fake_volume_id[0],
            'Mapped': 'true',
            'Total-filled-block': '100',
            'Creation-time': 'Wed, Jan 08 20:23:23 2020',
        }, {
            'LV-ID': self.fake_lv_id[0],
            'Mapping': '---',
            'Used': '20000',
            'Size': '20000',
            'ID': self.fake_partition_id[1],
            'Progress': '---',
            'Min-reserve': '20000',
            'Last-modification-time': 'Sat, Jan 11 22:18:40 2020',
            'Valid-filled-block': '100',
            'Name': self.fake_volume_id[1],
            'Mapped': 'false',
            'Total-filled-block': '100',
            'Creation-time': 'Sat, Jan 11 22:18:40 2020',
        }]
        if volume_id and pool_id:
            result.extend([{
                'LV-ID': pool_id,
                'Mapping': '---',
                'Used': '20000',
                'Size': '20000',
                'ID': self.fake_partition_id[2],
                'Progress': '---',
                'Min-reserve': '20000',
                'Last-modification-time': 'Sat, Jan 15 22:18:40 2020',
                'Valid-filled-block': '100',
                'Name': volume_id,
                'Mapped': 'false',
                'Total-filled-block': '100',
                'Creation-time': 'Sat, Jan 15 22:18:40 2020',
            }, {
                'LV-ID': '987654321',
                'Mapping': '---',
                'Used': '20000',
                'Size': '20000',
                'ID': '123123123123',
                'Progress': '---',
                'Min-reserve': '20000',
                'Last-modification-time': 'Sat, Jan 12 22:18:40 2020',
                'Valid-filled-block': '100',
                'Name': volume_id,
                'Mapped': 'false',
                'Total-filled-block': '100',
                'Creation-time': 'Sat, Jan 15 22:18:40 2020',
            }, {
                'LV-ID': self.fake_lv_id[0],
                'Mapping': '---',
                'Used': '20000',
                'Size': '20000',
                'ID': '6bb119a8-d25b-45a7-8d1b-88e127885666',
                'Progress': '---',
                'Min-reserve': '20000',
                'Last-modification-time': 'Sat, Jan 16 22:18:40 2020',
                'Valid-filled-block': '100',
                'Name': volume_id,
                'Mapped': 'false',
                'Total-filled-block': '100',
                'Creation-time': 'Sat, Jan 14 22:18:40 2020',
            }])
        return (0, result)

    def get_fake_show_partition_detail(self):
        msg = """
show part -l
 ID: %s
 Name: %s
 LV-ID: %s
 Size: 20000
 Used: 20000
 Min-reserve: 20000
 Creation-time: Wed, Jan 08 20:23:23 2020
 Last-modification-time: Wed, Jan 08 20:23:23 2020
 Valid-filled-block: 100
 Total-filled-block: 100
 Progress: ---
 Mapped: true
 Mapping: CH:1/ID:0/LUN:0, CH:1/ID:0/LUN:1, CH:4/ID:0/LUN:0

 ID: %s
 Name: %s
 LV-ID: %s
 Size: 20000
 Used: 20000
 Min-reserve: 20000
 Creation-time: Sat, Jan 11 22:18:40 2020
 Last-modification-time: Sat, Jan 11 22:18:40 2020
 Valid-filled-block: 100
 Total-filled-block: 100
 Progress: ---
 Mapped: false
 Mapping: ---

CLI: Successful: 3 partition(s) shown
Return: 0x0000

RAIDCmd:>
"""
        return msg % (self.fake_partition_id[0],
                      self.fake_volume_id[0],
                      self.fake_lv_id[0],
                      self.fake_partition_id[1],
                      self.fake_volume_id[1],
                      self.fake_lv_id[0])

    def get_test_show_replica_detail_for_migrate(
            self, src_part_id, dst_part_id, volume_id, status='Completed'):
        result = [{
            'Pair-ID': self.fake_pair_id[0],
            'Name': 'Cinder-Snapshot',
            'Source-Device': 'DEEC',
            'Source': src_part_id,
            'Source-Type': 'LV-Partition',
            'Source-Name': volume_id,
            'Source-LV': '5DE94FF775D81C30',
            'Source-VS': '2C482316298F7A4E',
            'Source-Mapped': 'Yes',
            'Target-Device': 'DEEC',
            'Target': dst_part_id,
            'Target-Type': 'LV-Partition',
            'Target-Name': volume_id,
            'Target-LV': '5DE94FF775D81C30',
            'Target-VS': '033EA1FA4EA193EB',
            'Target-Mapped': 'No',
            'Type': 'Copy',
            'Priority': 'Normal',
            'Timeout': '---',
            'Incremental': '---',
            'Compression': '---',
            'Status': status,
            'Progress': '---',
            'Created-time': '01/11/2020 22:20 PM',
            'Sync-commence-time': '01/11/2020 22:20 PM',
            'Split-time': '01/11/2020 22:20 PM',
            'Completed-time': '01/11/2020 22:21 PM',
            'Description': '---',
        }]
        return (0, result)

    def get_test_show_replica_detail_for_si_sync_pair(self):
        result = [{
            'Pair-ID': self.fake_pair_id[0],
            'Name': 'Cinder-Snapshot',
            'Source-Device': 'DEEC',
            'Source': self.fake_snapshot_id[0],
            'Source-Type': 'LV-Partition',
            'Source-Name': '',
            'Source-LV': '5DE94FF775D81C30',
            'Source-VS': '2C482316298F7A4E',
            'Source-Mapped': 'Yes',
            'Target-Device': 'DEEC',
            'Target': self.fake_partition_id[1],
            'Target-Type': 'LV-Partition',
            'Target-Name': '',
            'Target-LV': '5DE94FF775D81C30',
            'Target-VS': '033EA1FA4EA193EB',
            'Target-Mapped': 'No',
            'Type': 'Copy',
            'Priority': 'Normal',
            'Timeout': '---',
            'Incremental': '---',
            'Compression': '---',
            'Status': 'Copy',
            'Progress': '---',
            'Created-time': '01/11/2020 22:20 PM',
            'Sync-commence-time': '01/11/2020 22:20 PM',
            'Split-time': '01/11/2020 22:20 PM',
            'Completed-time': '01/11/2020 22:21 PM',
            'Description': '---',
        }]
        return (0, result)

    def get_test_show_replica_detail_for_sync_pair(self):
        result = [{
            'Pair-ID': self.fake_pair_id[0],
            'Name': 'Cinder-Snapshot',
            'Source-Device': 'DEEC',
            'Source': self.fake_partition_id[0],
            'Source-Type': 'LV-Partition',
            'Source-Name': self.fake_volume_id[0],
            'Source-LV': '5DE94FF775D81C30',
            'Source-VS': '2C482316298F7A4E',
            'Source-Mapped': 'Yes',
            'Target-Device': 'DEEC',
            'Target': self.fake_partition_id[1],
            'Target-Type': 'LV-Partition',
            'Target-Name': self.fake_volume_id[1],
            'Target-LV': '5DE94FF775D81C30',
            'Target-VS': '033EA1FA4EA193EB',
            'Target-Mapped': 'No',
            'Type': 'Copy',
            'Priority': 'Normal',
            'Timeout': '---',
            'Incremental': '---',
            'Compression': '---',
            'Status': 'Copy',
            'Progress': '---',
            'Created-time': '01/11/2020 22:20 PM',
            'Sync-commence-time': '01/11/2020 22:20 PM',
            'Split-time': '01/11/2020 22:20 PM',
            'Completed-time': '01/11/2020 22:21 PM',
            'Description': '---',
        }]
        return (0, result)

    def get_test_show_replica_detail(self):
        result = [{
            'Pair-ID': '4BF246E26966F015',
            'Name': 'Cinder-Snapshot',
            'Source-Device': 'DEEC',
            'Source': self.fake_partition_id[2],
            'Source-Type': 'LV-Partition',
            'Source-Name': 'Part-2',
            'Source-LV': '5DE94FF775D81C30',
            'Source-VS': '2C482316298F7A4E',
            'Source-Mapped': 'No',
            'Target-Device': 'DEEC',
            'Target': self.fake_partition_id[3],
            'Target-Type': 'LV-Partition',
            'Target-Name': 'Part-1-Copy',
            'Target-LV': '5DE94FF775D81C30',
            'Target-VS': '714B80F0335F6E52',
            'Target-Mapped': 'No',
            'Type': 'Copy',
            'Priority': 'Normal',
            'Timeout': '---',
            'Incremental': '---',
            'Compression': '---',
            'Status': 'Completed',
            'Progress': '---',
            'Created-time': '01/11/2020 22:20 PM',
            'Sync-commence-time': '01/11/2020 22:20 PM',
            'Split-time': '01/11/2020 22:20 PM',
            'Completed-time': '01/11/2020 22:21 PM',
            'Description': '---',
        }, {
            'Pair-ID': self.fake_pair_id[0],
            'Name': 'Cinder-Migrate',
            'Source-Device': 'DEEC',
            'Source': self.fake_partition_id[0],
            'Source-Type': 'LV-Partition',
            'Source-Name': self.fake_volume_id[0],
            'Source-LV': '5DE94FF775D81C30',
            'Source-VS': '2C482316298F7A4E',
            'Source-Mapped': 'Yes',
            'Target-Device': 'DEEC',
            'Target': self.fake_partition_id[1],
            'Target-Type': 'LV-Partition',
            'Target-Name': self.fake_volume_id[1],
            'Target-LV': '5DE94FF775D81C30',
            'Target-VS': '033EA1FA4EA193EB',
            'Target-Mapped': 'No',
            'Type': 'Mirror',
            'Priority': 'Normal',
            'Timeout': '---',
            'Incremental': '---',
            'Compression': '---',
            'Status': 'Mirror',
            'Progress': '---',
            'Created-time': '01/11/2020 22:20 PM',
            'Sync-commence-time': '01/11/2020 22:20 PM',
            'Split-time': '01/11/2020 22:20 PM',
            'Completed-time': '01/11/2020 22:21 PM',
            'Description': '---',
        }, {
            'Pair-ID': self.fake_pair_id[1],
            'Name': 'Cinder-Migrate',
            'Source-Device': 'DEEC',
            'Source': self.fake_partition_id[4],
            'Source-Type': 'LV-Partition',
            'Source-Name': self.fake_volume_id[0],
            'Source-LV': '5DE94FF775D81C30',
            'Source-VS': '2C482316298F7A4E',
            'Source-Mapped': 'No',
            'Target-Device': 'DEEC',
            'Target': self.fake_partition_id[5],
            'Target-Type': 'LV-Partition',
            'Target-Name': self.fake_volume_id[1],
            'Target-LV': '5DE94FF775D81C30',
            'Target-VS': '714B80F0335F6E52',
            'Target-Mapped': 'Yes',
            'Type': 'Mirror',
            'Priority': 'Normal',
            'Timeout': '---',
            'Incremental': '---',
            'Compression': '---',
            'Status': 'Mirror',
            'Progress': '---',
            'Created-time': '01/11/2020 22:20 PM',
            'Sync-commence-time': '01/11/2020 22:20 PM',
            'Split-time': '01/11/2020 22:20 PM',
            'Completed-time': '01/11/2020 22:21 PM',
            'Description': '---',
        }]
        return (0, result)

    def get_fake_show_replica_detail(self):
        msg = """
show replica -l
 Pair-ID: 4BF246E26966F015
 Name: Cinder-Snapshot
 Source-Device: DEEC
 Source: %s
 Source-Type: LV-Partition
 Source-Name: Part-2
 Source-LV: 5DE94FF775D81C30
 Source-VS: 2C482316298F7A4E
 Source-Mapped: No
 Target-Device: DEEC
 Target: %s
 Target-Type: LV-Partition
 Target-Name: Part-1-Copy
 Target-LV: 5DE94FF775D81C30
 Target-VS: 714B80F0335F6E52
 Target-Mapped: No
 Type: Copy
 Priority: Normal
 Timeout: ---
 Incremental: ---
 Compression: ---
 Status: Completed
 Progress: ---
 Created-time: 01/11/2020 22:20 PM
 Sync-commence-time: 01/11/2020 22:20 PM
 Split-time: 01/11/2020 22:20 PM
 Completed-time: 01/11/2020 22:21 PM
 Description: ---

 Pair-ID: %s
 Name: Cinder-Migrate
 Source-Device: DEEC
 Source: %s
 Source-Type: LV-Partition
 Source-Name: %s
 Source-LV: 5DE94FF775D81C30
 Source-VS: 2C482316298F7A4E
 Source-Mapped: Yes
 Target-Device: DEEC
 Target: %s
 Target-Type: LV-Partition
 Target-Name: %s
 Target-LV: 5DE94FF775D81C30
 Target-VS: 033EA1FA4EA193EB
 Target-Mapped: No
 Type: Mirror
 Priority: Normal
 Timeout: ---
 Incremental: ---
 Compression: ---
 Status: Mirror
 Progress: ---
 Created-time: 01/11/2020 22:20 PM
 Sync-commence-time: 01/11/2020 22:20 PM
 Split-time: 01/11/2020 22:20 PM
 Completed-time: 01/11/2020 22:21 PM
 Description: ---

 Pair-ID: %s
 Name: Cinder-Migrate
 Source-Device: DEEC
 Source: %s
 Source-Type: LV-Partition
 Source-Name: %s
 Source-LV: 5DE94FF775D81C30
 Source-VS: 2C482316298F7A4E
 Source-Mapped: No
 Target-Device: DEEC
 Target: %s
 Target-Type: LV-Partition
 Target-Name: %s
 Target-LV: 5DE94FF775D81C30
 Target-VS: 714B80F0335F6E52
 Target-Mapped: Yes
 Type: Mirror
 Priority: Normal
 Timeout: ---
 Incremental: ---
 Compression: ---
 Status: Mirror
 Progress: ---
 Created-time: 01/11/2020 22:20 PM
 Sync-commence-time: 01/11/2020 22:20 PM
 Split-time: 01/11/2020 22:20 PM
 Completed-time: 01/11/2020 22:21 PM
 Description: ---

CLI: Successful: 3 replication job(s) shown
Return: 0x0000

RAIDCmd:>
"""
        return msg % (self.fake_partition_id[2],
                      self.fake_partition_id[3],
                      self.fake_pair_id[0],
                      self.fake_partition_id[0],
                      self.fake_volume_id[0],
                      self.fake_partition_id[1],
                      self.fake_volume_id[1],
                      self.fake_pair_id[1],
                      self.fake_partition_id[4],
                      self.fake_volume_id[0],
                      self.fake_partition_id[5],
                      self.fake_volume_id[1])

    def get_test_show_lv(self):
        return (0, [{
            'Name': 'LV-1',
            'LD-amount': '1',
            'Available': '841978 MB',
            'ID': self.fake_lv_id[0],
            'Progress': '---',
            'Size': '857982 MB',
            'Status': 'On-line',
        }])

    def get_fake_show_lv(self):
        msg = """
show lv
 ID  Name  LD-amount  Size       Available  Progress  Status
--------------------------------------------------------------
 %s  LV-1  1          857982 MB  841978 MB  ---       On-line

CLI: Successful: 1 Logical Volumes(s) shown
Return: 0x0000

RAIDCmd:>
"""
        return msg % self.fake_lv_id[0]

    def get_test_show_lv_detail(self):
        return (0, [{
            'Policy': 'Default',
            'Status': 'On-line',
            'ID': self.fake_lv_id[0],
            'Available': '841978 MB',
            'Expandable-size': '0 MB',
            'Name': 'LV-1',
            'Size': '857982 MB',
            'LD-amount': '1',
            'Progress': '---',
        }])

    def get_fake_show_lv_detail(self):
        msg = """
show lv -l
 ID: %s
 Name: LV-1
 LD-amount: 1
 Size: 857982 MB
 Available: 841978 MB
 Expandable-size: 0 MB
 Policy: Default
 Progress: ---
 Status: On-line

CLI: Successful: 1 Logical Volumes(s) shown
Return: 0x0000

RAIDCmd:>
"""
        return msg % self.fake_lv_id[0]

    def get_test_show_lv_tier_for_migration(self):
        return (0, [{
            'LV-Name': 'LV-1',
            'LV-ID': self.fake_lv_id[1],
            'Tier': '0',
            'Size': '418.93 GB',
            'Used': '10 GB(2.4%)',
            'Data Service': '0 MB(0.0%)',
            'Reserved Ratio': '10.0%',
        }, {
            'LV-Name': 'LV-1',
            'LV-ID': self.fake_lv_id[1],
            'Tier': '3',
            'Size': '931.02 GB',
            'Used': '0 MB(0.0%)',
            'Data Service': '0 MB(0.0%)',
            'Reserved Ratio': '0.0%',
        }])

    def get_test_show_lv_tier(self):
        return (0, [{
            'LV-Name': 'LV-1',
            'LV-ID': self.fake_lv_id[0],
            'Tier': '0',
            'Size': '418.93 GB',
            'Used': '10 GB(2.4%)',
            'Data Service': '0 MB(0.0%)',
            'Reserved Ratio': '10.0%',
        }, {
            'LV-Name': 'LV-1',
            'LV-ID': self.fake_lv_id[0],
            'Tier': '3',
            'Size': '931.02 GB',
            'Used': '0 MB(0.0%)',
            'Data Service': '0 MB(0.0%)',
            'Reserved Ratio': '0.0%',
        }])

    def get_fake_show_lv_tier(self):
        msg = """
show lv tier
 LV-Name  LV-ID  Tier  Size       Used          Data Service   Reserved Ratio
------------------------------------------------------------------------------
 LV-1     %s     0     418.93 GB  10 GB(2.4%%)  0 MB(0.0%%)    10.0%%
 LV-1     %s     3     931.02 GB  0 MB(0.0%%)   0 MB(0.0%%)    0.0%%

CLI: Successful: 2 lv tiering(s) shown
Return: 0x0000

RAIDCmd:>
"""
        return msg % (self.fake_lv_id[0],
                      self.fake_lv_id[0])

    def get_test_show_device(self):
        return (0, [{
            'ID': self.fake_system_id[0],
            'Connected-IP': self.fake_manage_port_ip[0],
            'Name': '---',
            'Index': '0*',
            'JBOD-ID': 'N/A',
            'Capacity': '1.22 TB',
            'Model': self.fake_model[0],
            'Service-ID': '8445676',
        }])

    def get_fake_show_device(self):
        msg = """
show device
 Index  ID     Model  Name  Connected-IP  JBOD-ID  Capacity  Service-ID
------------------------------------------------------------------------
 0*     %s     %s     ---   %s            N/A      1.22 TB   8445676

CLI: Successful: 1 device(s) found
Return: 0x0000

RAIDCmd:>
"""
        return msg % (self.fake_system_id[0],
                      self.fake_model[0],
                      self.fake_manage_port_ip[0])

    def get_test_show_channel_single(self):
        return (0, [{
            'ID': '112',
            'defClock': 'Auto',
            'Type': 'FIBRE',
            'Mode': 'Host',
            'Width': '---',
            'Ch': '0',
            'MCS': 'N/A',
            'curClock': '---',
        }, {
            'ID': '0',
            'defClock': 'Auto',
            'Type': 'NETWORK',
            'Mode': 'Host',
            'Width': 'iSCSI',
            'Ch': '1',
            'MCS': '0',
            'curClock': '---',
        }])

    def get_test_show_channel_with_mcs(self):
        return (0, [{
            'ID': '112',
            'defClock': 'Auto',
            'Type': 'FIBRE',
            'Mode': 'Host',
            'Width': '---',
            'Ch': '0',
            'MCS': 'N/A',
            'curClock': '---',
        }, {
            'ID': '0',
            'defClock': 'Auto',
            'Type': 'NETWORK',
            'Mode': 'Host',
            'Width': 'iSCSI',
            'Ch': '1',
            'MCS': '1',
            'curClock': '---',
        }, {
            'ID': '0',
            'defClock': 'Auto',
            'Type': 'NETWORK',
            'Mode': 'Host',
            'Width': 'iSCSI',
            'Ch': '2',
            'MCS': '1',
            'curClock': '---',
        }, {
            'ID': '---',
            'defClock': '6.0 Gbps',
            'Type': 'SAS',
            'Mode': 'Drive',
            'Width': 'SAS',
            'Ch': '3',
            'MCS': 'N/A',
            'curClock': '6.0 Gbps',
        }, {
            'ID': '0',
            'defClock': 'Auto',
            'Type': 'NETWORK',
            'Mode': 'Host',
            'Width': 'iSCSI',
            'Ch': '4',
            'MCS': '2',
            'curClock': '---',
        }, {
            'ID': '112',
            'defClock': 'Auto',
            'Type': 'FIBRE',
            'Mode': 'Host',
            'Width': '---',
            'Ch': '5',
            'MCS': 'N/A',
            'curClock': '---',
        }])

    def get_test_show_channel_without_mcs(self):
        return (0, [{
            'ID': '112',
            'defClock': 'Auto',
            'Type': 'FIBRE',
            'Mode': 'Host',
            'Width': '---',
            'Ch': '0',
            'curClock': '---',
        }, {
            'ID': '0',
            'defClock': 'Auto',
            'Type': 'NETWORK',
            'Mode': 'Host',
            'Width': 'iSCSI',
            'Ch': '1',
            'curClock': '---',
        }, {
            'ID': '0',
            'defClock': 'Auto',
            'Type': 'NETWORK',
            'Mode': 'Host',
            'Width': 'iSCSI',
            'Ch': '2',
            'curClock': '---',
        }, {
            'ID': '---',
            'defClock': '6.0 Gbps',
            'Type': 'SAS',
            'Mode': 'Drive',
            'Width': 'SAS',
            'Ch': '3',
            'curClock': '6.0 Gbps',
        }, {
            'ID': '0',
            'defClock': 'Auto',
            'Type': 'NETWORK',
            'Mode': 'Host',
            'Width': 'iSCSI',
            'Ch': '4',
            'curClock': '---',
        }, {
            'ID': '112',
            'defClock': 'Auto',
            'Type': 'FIBRE',
            'Mode': 'Host',
            'Width': '---',
            'Ch': '5',
            'curClock': '---',
        }])

    def get_test_show_channel_with_diff_target_id(self):
        return (0, [{
            'ID': '32',
            'defClock': 'Auto',
            'Type': 'FIBRE',
            'Mode': 'Host',
            'Width': '---',
            'Ch': '0',
            'MCS': 'N/A',
            'curClock': '---',
        }, {
            'ID': '0',
            'defClock': 'Auto',
            'Type': 'NETWORK',
            'Mode': 'Host',
            'Width': 'iSCSI',
            'Ch': '1',
            'MCS': '0',
            'curClock': '---',
        }, {
            'ID': '0',
            'defClock': 'Auto',
            'Type': 'NETWORK',
            'Mode': 'Host',
            'Width': 'iSCSI',
            'Ch': '2',
            'MCS': '1',
            'curClock': '---',
        }, {
            'ID': '---',
            'defClock': '6.0 Gbps',
            'Type': 'SAS',
            'Mode': 'Drive',
            'Width': 'SAS',
            'Ch': '3',
            'MCS': 'N/A',
            'curClock': '6.0 Gbps',
        }, {
            'ID': '0',
            'defClock': 'Auto',
            'Type': 'NETWORK',
            'Mode': 'Host',
            'Width': 'iSCSI',
            'Ch': '4',
            'MCS': '2',
            'curClock': '---',
        }, {
            'ID': '48',
            'defClock': 'Auto',
            'Type': 'FIBRE',
            'Mode': 'Host',
            'Width': '---',
            'Ch': '5',
            'MCS': 'N/A',
            'curClock': '---',
        }])

    def get_test_show_channel(self):
        return (0, [{
            'ID': '112',
            'defClock': 'Auto',
            'Type': 'FIBRE',
            'Mode': 'Host',
            'Width': '---',
            'Ch': '0',
            'MCS': 'N/A',
            'curClock': '---',
        }, {
            'ID': '0',
            'defClock': 'Auto',
            'Type': 'NETWORK',
            'Mode': 'Host',
            'Width': 'iSCSI',
            'Ch': '1',
            'MCS': '0',
            'curClock': '---',
        }, {
            'ID': '0',
            'defClock': 'Auto',
            'Type': 'NETWORK',
            'Mode': 'Host',
            'Width': 'iSCSI',
            'Ch': '2',
            'MCS': '1',
            'curClock': '---',
        }, {
            'ID': '---',
            'defClock': '6.0 Gbps',
            'Type': 'SAS',
            'Mode': 'Drive',
            'Width': 'SAS',
            'Ch': '3',
            'MCS': 'N/A',
            'curClock': '6.0 Gbps',
        }, {
            'ID': '0',
            'defClock': 'Auto',
            'Type': 'NETWORK',
            'Mode': 'Host',
            'Width': 'iSCSI',
            'Ch': '4',
            'MCS': '2',
            'curClock': '---',
        }, {
            'ID': '112',
            'defClock': 'Auto',
            'Type': 'FIBRE',
            'Mode': 'Host',
            'Width': '---',
            'Ch': '5',
            'MCS': 'N/A',
            'curClock': '---',
        }])

    def get_fake_show_channel(self):
        msg = """
show ch
 Ch  Mode   Type     defClock  curClock  Width  ID   MCS
---------------------------------------------------------
 0   Host   FIBRE    Auto      ---       ---    112  N/A
 1   Host   NETWORK  Auto      ---       iSCSI  0    0
 2   Host   NETWORK  Auto      ---       iSCSI  0    1
 3   Drive  SAS      6.0 Gbps  6.0 Gbps  SAS    ---  N/A
 4   Host   NETWORK  Auto      ---       iSCSI  0    2
 5   Host   FIBRE    Auto      ---       ---    112  N/A

CLI: Successful: : 6 channel(s) shown
Return: 0x0000

RAIDCmd:>
"""
        return msg

    def get_test_show_channel_r_model_diff_target_id(self):
        return (0, [{
            'Mode': 'Host',
            'AID': '32',
            'defClock': 'Auto',
            'MCS': 'N/A',
            'Ch': '0',
            'BID': '33',
            'curClock': '---',
            'Width': '---',
            'Type': 'FIBRE',
        }, {
            'Mode': 'Host',
            'AID': '0',
            'defClock': 'Auto',
            'MCS': '0',
            'Ch': '1',
            'BID': '1',
            'curClock': '---',
            'Width': 'iSCSI',
            'Type': 'NETWORK',
        }, {
            'Mode': 'Host',
            'AID': '0',
            'defClock': 'Auto',
            'MCS': '1',
            'Ch': '2',
            'BID': '1',
            'curClock': '---',
            'Width': 'iSCSI',
            'Type': 'NETWORK',
        }, {
            'Mode': 'Drive',
            'AID': '---',
            'defClock': '6.0 Gbps',
            'MCS': 'N/A',
            'Ch': '3',
            'BID': '---',
            'curClock': '6.0 Gbps',
            'Width': 'SAS',
            'Type': 'SAS',
        }, {
            'Mode': 'Host',
            'AID': '0',
            'defClock': 'Auto',
            'MCS': '2',
            'Ch': '4',
            'BID': '1',
            'curClock': '---',
            'Width': 'iSCSI',
            'Type': 'NETWORK',
        }, {
            'Mode': 'Host',
            'AID': '48',
            'defClock': 'Auto',
            'MCS': 'N/A',
            'Ch': '5',
            'BID': '49',
            'curClock': '---',
            'Width': '---',
            'Type': 'FIBRE',
        }])

    def get_test_show_channel_r_model(self):
        return (0, [{
            'Mode': 'Host',
            'AID': '112',
            'defClock': 'Auto',
            'MCS': 'N/A',
            'Ch': '0',
            'BID': '113',
            'curClock': '---',
            'Width': '---',
            'Type': 'FIBRE',
        }, {
            'Mode': 'Host',
            'AID': '0',
            'defClock': 'Auto',
            'MCS': '0',
            'Ch': '1',
            'BID': '1',
            'curClock': '---',
            'Width': 'iSCSI',
            'Type': 'NETWORK',
        }, {
            'Mode': 'Host',
            'AID': '0',
            'defClock': 'Auto',
            'MCS': '1',
            'Ch': '2',
            'BID': '1',
            'curClock': '---',
            'Width': 'iSCSI',
            'Type': 'NETWORK',
        }, {
            'Mode': 'Drive',
            'AID': '---',
            'defClock': '6.0 Gbps',
            'MCS': 'N/A',
            'Ch': '3',
            'BID': '---',
            'curClock': '6.0 Gbps',
            'Width': 'SAS',
            'Type': 'SAS',
        }, {
            'Mode': 'Host',
            'AID': '0',
            'defClock': 'Auto',
            'MCS': '2',
            'Ch': '4',
            'BID': '1',
            'curClock': '---',
            'Width': 'iSCSI',
            'Type': 'NETWORK',
        }, {
            'Mode': 'Host',
            'AID': '112',
            'defClock': 'Auto',
            'MCS': 'N/A',
            'Ch': '5',
            'BID': '113',
            'curClock': '---',
            'Width': '---',
            'Type': 'FIBRE',
        }])

    def get_fake_show_channel_r_model(self):
        msg = """
show ch
 Ch    Mode   Type     defClock  curClock  Width  AID  BID  MCS
----------------------------------------------------------------
 0     Host   FIBRE    Auto      ---       ---    112  113  N/A
 1     Host   NETWORK  Auto      ---       iSCSI  0    1    0
 2     Host   NETWORK  Auto      ---       iSCSI  0    1    1
 3     Drive  SAS      6.0 Gbps  6.0 Gbps  SAS    ---  ---  N/A
 4     Host   NETWORK  Auto      ---       iSCSI  0    1    2
 5     Host   FIBRE    Auto      ---       ---    112  113  N/A

CLI: Successful: : 9 channel(s) shown
Return: 0x0000

RAIDCmd:>
"""
        return msg

    def get_show_map_with_lun_map_on_zoning(self):
        return (0, [{
            'Ch': '0',
            'LUN': '0',
            'Media': 'PART',
            'Host-ID': self.fake_initiator_wwpns[0],
            'Target': '112',
            'Name': 'Part-1',
            'ID': self.fake_partition_id[0],
        }])

    def get_test_show_map(self, partition_id=None, channel_id=None):
        if partition_id and channel_id:
            return (0, [{
                'Ch': channel_id,
                'LUN': '0',
                'Media': 'PART',
                'Host-ID': '---',
                'Target': '0',
                'Name': 'Part-1',
                'ID': partition_id,
            }, {
                'Ch': channel_id,
                'LUN': '1',
                'Media': 'PART',
                'Host-ID': '---',
                'Target': '0',
                'Name': 'Part-1',
                'ID': partition_id,
            }])
        else:
            return (0, [{
                'Ch': '1',
                'LUN': '0',
                'Media': 'PART',
                'Host-ID': self.fake_initiator_iqn[0],
                'Target': '0',
                'Name': 'Part-1',
                'ID': self.fake_partition_id[0],
            }, {
                'Ch': '1',
                'LUN': '1',
                'Media': 'PART',
                'Host-ID': self.fake_initiator_iqn[0],
                'Target': '0',
                'Name': 'Part-1',
                'ID': self.fake_partition_id[0],
            }, {
                'Ch': '4',
                'LUN': '0',
                'Media': 'PART',
                'Host-ID': self.fake_initiator_iqn[0],
                'Target': '0',
                'Name': 'Part-1',
                'ID': self.fake_partition_id[0],
            }])

    def get_test_show_map_fc(self):
        return (0, [{
            'Ch': '0',
            'LUN': '0',
            'Media': 'PART',
            'Host-ID': self.fake_initiator_wwpns[0],
            'Target': '112',
            'Name': 'Part-1',
            'ID': self.fake_partition_id[0],
        }, {
            'Ch': '0',
            'LUN': '0',
            'Media': 'PART',
            'Host-ID': self.fake_initiator_wwpns[1],
            'Target': '112',
            'Name': 'Part-1',
            'ID': self.fake_partition_id[0],
        }, {
            'Ch': '5',
            'LUN': '0',
            'Media': 'PART',
            'Host-ID': self.fake_initiator_wwpns[0],
            'Target': '112',
            'Name': 'Part-1',
            'ID': self.fake_partition_id[0],
        }, {
            'Ch': '5',
            'LUN': '0',
            'Media': 'PART',
            'Host-ID': self.fake_initiator_wwpns[1],
            'Target': '112',
            'Name': 'Part-1',
            'ID': self.fake_partition_id[0],
        }])

    def get_test_show_map_multimap(self):
        return (0, [{
            'Ch': '1',
            'LUN': '0',
            'Media': 'PART',
            'Host-ID': '---',
            'Target': '0',
            'Name': 'Part-1',
            'ID': self.fake_partition_id[0],
        }, {
            'Ch': '1',
            'LUN': '1',
            'Media': 'PART',
            'Host-ID': '---',
            'Target': '0',
            'Name': 'Part-1',
            'ID': self.fake_partition_id[0],
        }, {
            'Ch': '4',
            'LUN': '0',
            'Media': 'PART',
            'Host-ID': '210000E08B0AADE1',
            'Target': '0',
            'Name': 'Part-1',
            'ID': self.fake_partition_id[0],
        }, {
            'Ch': '4',
            'LUN': '0',
            'Media': 'PART',
            'Host-ID': '210000E08B0AADE2',
            'Target': '0',
            'Name': 'Part-1',
            'ID': self.fake_partition_id[0],
        }])

    def get_fake_show_map(self):
        msg = """
show map
 Ch  Target  LUN  Media  Name    ID  Host-ID
-----------------------------------------------------------
 1   0       0    PART   Part-1  %s  %s
 1   0       1    PART   Part-1  %s  %s
 4   0       0    PART   Part-1  %s  %s

CLI: Successful: 3 mapping(s) shown
Return: 0x0000

RAIDCmd:>
"""
        return msg % (self.fake_partition_id[0],
                      self.fake_initiator_iqn[0],
                      self.fake_partition_id[0],
                      self.fake_initiator_iqn[0],
                      self.fake_partition_id[0],
                      self.fake_initiator_iqn[0])

    def get_test_show_license_full(self):
        return (0, {
            'Local Volume Copy': {
                'Support': False,
                'Amount': '8/256',
            },
            'Synchronous Remote Mirror': {
                'Support': False,
                'Amount': '8/256',
            },
            'Snapshot': {
                'Support': False,
                'Amount': '1024/16384',
            },
            'Self-Encryption Drives': {
                'Support': False,
                'Amount': '---',
            },
            'Compression': {
                'Support': False,
                'Amount': '---',
            },
            'Local volume Mirror': {
                'Support': False,
                'Amount': '8/256',
            },
            'Storage Tiering': {
                'Support': False,
                'Amount': '---',
            },
            'Asynchronous Remote Mirror': {
                'Support': False,
                'Amount': '8/256',
            },
            'Scale-out': {
                'Support': False,
                'Amount': 'Not Support',
            },
            'Thin Provisioning': {
                'Support': False,
                'Amount': '---',
            },
            'Max JBOD': {
                'Support': False,
                'Amount': '15',
            },
            'EonPath': {
                'Support': False,
                'Amount': '---',
            }
        })

    def get_test_show_license_thin(self):
        return (0, {
            'Local Volume Copy': {
                'Support': False,
                'Amount': '8/256',
            },
            'Synchronous Remote Mirror': {
                'Support': False,
                'Amount': '8/256',
            },
            'Snapshot': {
                'Support': False,
                'Amount': '1024/16384',
            },
            'Self-Encryption Drives': {
                'Support': False,
                'Amount': '---',
            },
            'Compression': {
                'Support': False,
                'Amount': '---',
            },
            'Local volume Mirror': {
                'Support': False,
                'Amount': '8/256',
            },
            'Storage Tiering': {
                'Support': False,
                'Amount': '---',
            },
            'Asynchronous Remote Mirror': {
                'Support': False,
                'Amount': '8/256',
            },
            'Scale-out': {
                'Support': False,
                'Amount': 'Not Support',
            },
            'Thin Provisioning': {
                'Support': True,
                'Amount': '---',
            },
            'Max JBOD': {
                'Support': False,
                'Amount': '15',
            },
            'EonPath': {
                'Support': False,
                'Amount': '---',
            }
        })

    def get_fake_show_license(self):
        msg = """
show license
 License                     Amount(Partition/Subsystem)  Expired
------------------------------------------------------------------
 EonPath                     ---                          Expired
 Scale-out                   Not Support                  ---
 Snapshot                    1024/16384                   Expired
 Local Volume Copy           8/256                        Expired
 Local volume Mirror         8/256                        Expired
 Synchronous Remote Mirror   8/256                        Expired
 Asynchronous Remote Mirror  8/256                        Expired
 Compression                 ---                          Expired
 Thin Provisioning           ---                          Expired
 Storage Tiering             ---                          Expired
 Max JBOD                    15                           Expired
 Self-Encryption Drives      ---                          Expired

CLI: Successful
Return: 0x0000

RAIDCmd:>
"""
        return msg

    def get_test_show_wwn_with_g_model(self):
        return (0, [{
            'ID': 'ID:112',
            'WWPN': self.fake_target_wwpns[0],
            'CH': '0',
            'WWNN': self.fake_target_wwnns[0],
        }, {
            'ID': 'ID:112',
            'WWPN': self.fake_target_wwpns[1],
            'CH': '5',
            'WWNN': self.fake_target_wwnns[0],
        }])

    def get_test_show_wwn_with_diff_target_id(self):
        return (0, [{
            'ID': 'AID:32',
            'WWPN': self.fake_target_wwpns[0],
            'CH': '0',
            'WWNN': self.fake_target_wwnns[0],
        }, {
            'ID': 'BID:33',
            'WWPN': self.fake_target_wwpns[2],
            'CH': '0',
            'WWNN': self.fake_target_wwnns[1],
        }, {
            'ID': 'AID:48',
            'WWPN': self.fake_target_wwpns[1],
            'CH': '5',
            'WWNN': self.fake_target_wwnns[0],
        }, {
            'ID': 'BID:49',
            'WWPN': self.fake_target_wwpns[3],
            'CH': '5',
            'WWNN': self.fake_target_wwnns[1],
        }])

    def get_test_show_wwn(self):
        return (0, [{
            'ID': 'AID:112',
            'WWPN': self.fake_target_wwpns[0],
            'CH': '0',
            'WWNN': self.fake_target_wwnns[0],
        }, {
            'ID': 'BID:113',
            'WWPN': self.fake_target_wwpns[2],
            'CH': '0',
            'WWNN': self.fake_target_wwnns[1],
        }, {
            'ID': 'AID:112',
            'WWPN': self.fake_target_wwpns[1],
            'CH': '5',
            'WWNN': self.fake_target_wwnns[0],
        }, {
            'ID': 'BID:113',
            'WWPN': self.fake_target_wwpns[3],
            'CH': '5',
            'WWNN': self.fake_target_wwnns[1],
        }])

    def get_fake_show_wwn(self):
        msg = """
show wwn
WWN entries in controller for host channels:
 CH  ID       WWNN  WWPN
-------------------------------------------------
 0   AID:112  %s    %s
 0   BID:113  %s    %s
 5   AID:112  %s    %s
 5   BID:113  %s    %s

CLI: Successful
Return: 0x0000

RAIDCmd:>
"""
        return msg % (self.fake_target_wwnns[0], self.fake_target_wwpns[0],
                      self.fake_target_wwnns[1], self.fake_target_wwpns[2],
                      self.fake_target_wwnns[0], self.fake_target_wwpns[1],
                      self.fake_target_wwnns[1], self.fake_target_wwpns[3])

    def get_test_show_iqn(self):
        return (0, [{
            'Name': self.fake_initiator_iqn[0][-16:],
            'IQN': self.fake_initiator_iqn[0],
            'User': '---',
            'Password': '---',
            'Target': '---',
            'Target-Password': '---',
            'IP': '0.0.0.0',
            'Mask': '0.0.0.0',
        }])

    def get_fake_show_iqn(self):
        msg = """
show iqn
Detected host IQN:
 IQN
----------------------------------------
 %s


List of initiator IQN(s):
--------------------------
 Name: %s
 IQN: %s
 User: ---
 Password: ---
 Target: ---
 Target-Password: ---
 IP: 0.0.0.0
 Mask: 0.0.0.0

CLI: Successful: 1 initiator iqn(s) shown
Return: 0x0000

RAIDCmd:>
"""
        return msg % (self.fake_initiator_iqn[0],
                      self.fake_initiator_iqn[0][-16:],
                      self.fake_initiator_iqn[0])

    def get_test_show_host(self):
        return (0, [{
            'Fibre connection option': 'Point to point only',
            'Max queued count': '1024',
            'Max LUN per ID': '64',
            'CHAP': 'Disabled',
            'Jumbo frame': 'Disabled',
            'Max concurrent LUN connection': '4',
            'LUN connection reserved tags': '4',
            'Peripheral device type': 'No Device Present (Type=0x7f)',
            'Peripheral device qualifier': 'Connected',
            'Removable media support': 'Disabled',
            'LUN applicability': 'First Undefined LUN',
            'Supported CHS Cylinder': 'Variable',
            'Supported CHS Head': 'Variable',
            'Supported CHS Sector': 'Variable',
        }])

    def get_fake_show_host(self):
        msg = """
show host
 Fibre connection option: Point to point only
 Max queued count: 1024
 Max LUN per ID: 64
 CHAP: Disabled
 Jumbo frame: Disabled
 Max concurrent LUN connection: 4
 LUN connection reserved tags: 4
 Peripheral device type: No Device Present (Type=0x7f)
 Peripheral device qualifier: Connected
 Removable media support: Disabled
 LUN applicability: First Undefined LUN
 Supported CHS Cylinder: Variable
 Supported CHS Head: Variable
 Supported CHS Sector: Variable

CLI: Successful
Return: 0x0000

RAIDCmd:>
"""
        return msg

    def get_fake_discovery(self, target_iqns, target_portals):
        template = '%s,1 %s'

        if len(target_iqns) == 1:
            result = template % (target_portals[0], target_iqns[0])
            return (0, result)

        result = []
        for i in range(len(target_iqns)):
            result.append(template % (
                target_portals[i], target_iqns[i]))
        return (0, '\n'.join(result))

    class Fake_cinder_object(object):
        id = None

        def __init__(self, test_volume):
            self.id = test_volume

    class Fake_cinder_snapshot(Fake_cinder_object):
        provider_location = None

        def __init__(self, id, provider_location):
            self.id = id
            self.provider_location = provider_location

    fake_cinder_volumes = [Fake_cinder_object(test_dst_volume['id'])]
    fake_cinder_snapshots = [Fake_cinder_object(fake_snapshot_name[1])]


class InfortrendCLITestCase(test.TestCase):

    CommandList = ['CreateLD', 'CreateLV',
                   'CreatePartition', 'DeletePartition',
                   'CreateMap', 'DeleteMap',
                   'CreateSnapshot', 'DeleteSnapshot',
                   'CreateReplica', 'DeleteReplica',
                   'CreateIQN', 'DeleteIQN',
                   'ShowLD', 'ShowLV',
                   'ShowPartition', 'ShowSnapshot',
                   'ShowDevice', 'ShowChannel',
                   'ShowDisk', 'ShowMap',
                   'ShowNet', 'ShowLicense',
                   'ShowWWN', 'ShowReplica',
                   'ShowIQN', 'ShowHost', 'ConnectRaid',
                   'SetPartition', 'SetLV']

    def __init__(self, *args, **kwargs):
        super(InfortrendCLITestCase, self).__init__(*args, **kwargs)
        self.cli_data = InfortrendCLITestData()

    def _cli_set(self, cli, fake_result):
        cli_conf = {
            'path': '',
            'password': '',
            'ip': '',
            'cli_retry_time': 1,
            'raidcmd_timeout': 60,
            'cli_cache': False,
            'pid': 12345,
            'fd': 10,
        }
        cli = cli(cli_conf)

        cli._execute = mock.Mock(return_value=fake_result)

        return cli

    def _cli_multi_set(self, cli, fake_result_list):
        cli_conf = {
            'path': '',
            'password': '',
            'ip': '',
            'cli_retry_time': 5,
            'raidcmd_timeout': 60,
            'cli_cache': False,
            'pid': 12345,
            'fd': 10,
        }
        cli = cli(cli_conf)

        cli._execute = mock.Mock(side_effect=fake_result_list)

        return cli

    def _test_command_succeed(self, command):

        fake_cli_succeed = self.cli_data.get_fake_cli_succeed()
        test_command = self._cli_set(command, fake_cli_succeed)

        rc, out = test_command.execute()
        self.assertEqual(0, rc)

    def _test_command_failed(self, command):

        fake_cli_failed = self.cli_data.get_fake_cli_failed()
        test_command = self._cli_set(command, fake_cli_failed)

        rc, out = test_command.execute()
        self.assertEqual(int('0x000c', 16), rc)

    def _test_command_failed_retry_succeed(self, log_error, command):

        log_error.reset_mock()

        LOG_ERROR_STR = (
            'Retry %(retry)s times: %(method)s Failed %(rc)s: %(reason)s'
        )

        fake_result_list = [
            self.cli_data.get_fake_cli_failed(),
            self.cli_data.get_fake_cli_failed_with_network(),
            self.cli_data.get_fake_cli_succeed(),
        ]
        test_command = self._cli_multi_set(command, fake_result_list)

        rc, out = test_command.execute()
        self.assertEqual(11, rc)

        expect_log_error = [
            mock.call(LOG_ERROR_STR, {
                'retry': 1,
                'method': test_command.__class__.__name__,
                'rc': int('0x000c', 16),
                'reason': 'No selected device',
            }),
            mock.call(LOG_ERROR_STR, {
                'retry': 2,
                'method': test_command.__class__.__name__,
                'rc': int('0x000b', 16),
                'reason': 'Not exist: There is no such partition: 3345678',
            })
        ]
        log_error.assert_has_calls(expect_log_error)

    def _test_command_failed_retry_timeout(self, log_error, command):

        log_error.reset_mock()

        LOG_ERROR_STR = (
            'Retry %(retry)s times: %(method)s Failed %(rc)s: %(reason)s'
        )

        fake_result_list = [
            self.cli_data.get_fake_cli_failed(),
            self.cli_data.get_fake_cli_failed(),
            self.cli_data.get_fake_cli_failed(),
            self.cli_data.get_fake_cli_failed(),
            self.cli_data.get_fake_cli_failed(),
        ]
        test_command = self._cli_multi_set(command, fake_result_list)

        rc, out = test_command.execute()
        self.assertEqual(int('0x000c', 16), rc)
        self.assertEqual('No selected device', out)

        expect_log_error = [
            mock.call(LOG_ERROR_STR, {
                'retry': 1,
                'method': test_command.__class__.__name__,
                'rc': int('0x000c', 16),
                'reason': 'No selected device',
            }),
            mock.call(LOG_ERROR_STR, {
                'retry': 2,
                'method': test_command.__class__.__name__,
                'rc': int('0x000c', 16),
                'reason': 'No selected device',
            }),
            mock.call(LOG_ERROR_STR, {
                'retry': 3,
                'method': test_command.__class__.__name__,
                'rc': int('0x000c', 16),
                'reason': 'No selected device',
            }),
            mock.call(LOG_ERROR_STR, {
                'retry': 4,
                'method': test_command.__class__.__name__,
                'rc': int('0x000c', 16),
                'reason': 'No selected device',
            }),
            mock.call(LOG_ERROR_STR, {
                'retry': 5,
                'method': test_command.__class__.__name__,
                'rc': int('0x000c', 16),
                'reason': 'No selected device',
            })
        ]
        log_error.assert_has_calls(expect_log_error)

    def _test_show_command(self, fake_data, test_data, command, *params):

        test_command = self._cli_set(command, fake_data)

        rc, out = test_command.execute(*params)

        self.assertEqual(test_data[0], rc)

        if isinstance(out, list):
            for i in range(len(test_data[1])):
                self.assertDictEqual(test_data[1][i], out[i])
        else:
            self.assertDictEqual(test_data[1], out)

    @mock.patch.object(cli.LOG, 'debug', mock.Mock())
    def test_cli_all_command_execute(self):

        for command in self.CommandList:
            self._test_command_succeed(getattr(cli, command))
            self._test_command_failed(getattr(cli, command))

    @mock.patch.object(cli.LOG, 'error')
    def test_cli_all_command_execute_retry_succeed(self, log_error):

        for command in self.CommandList:
            self._test_command_failed_retry_succeed(
                log_error, getattr(cli, command))

    @mock.patch.object(cli.LOG, 'error')
    def test_cli_all_command_execute_retry_timeout(self, log_error):

        for command in self.CommandList:
            self._test_command_failed_retry_timeout(
                log_error, getattr(cli, command))

    @mock.patch.object(cli.LOG, 'debug', mock.Mock())
    def test_show_snapshot(self):
        self._test_show_command(
            self.cli_data.get_fake_show_snapshot(),
            self.cli_data.get_test_show_snapshot(),
            cli.ShowSnapshot)

    @mock.patch.object(cli.LOG, 'debug', mock.Mock())
    def test_show_snapshot_detail(self):
        self._test_show_command(
            self.cli_data.get_fake_show_snapshot_detail(),
            self.cli_data.get_test_show_snapshot_detail(),
            cli.ShowSnapshot, '-l')

    @mock.patch.object(cli.LOG, 'debug', mock.Mock())
    def test_show_net(self):
        self._test_show_command(
            self.cli_data.get_fake_show_net(),
            self.cli_data.get_test_show_net(),
            cli.ShowNet)

    @mock.patch.object(cli.LOG, 'debug', mock.Mock())
    def test_show_detail_net(self):
        self._test_show_command(
            self.cli_data.get_fake_show_net_detail(),
            self.cli_data.get_test_show_net_detail(),
            cli.ShowNet, '-l')

    @mock.patch.object(cli.LOG, 'debug', mock.Mock())
    def test_show_partition(self):
        self._test_show_command(
            self.cli_data.get_fake_show_partition(),
            self.cli_data.get_test_show_partition(),
            cli.ShowPartition)

    @mock.patch.object(cli.LOG, 'debug', mock.Mock())
    def test_show_partition_detail(self):
        self._test_show_command(
            self.cli_data.get_fake_show_partition_detail(),
            self.cli_data.get_test_show_partition_detail(),
            cli.ShowPartition, '-l')

    @mock.patch.object(cli.LOG, 'debug', mock.Mock())
    def test_show_lv(self):
        self._test_show_command(
            self.cli_data.get_fake_show_lv(),
            self.cli_data.get_test_show_lv(),
            cli.ShowLV)

    @mock.patch.object(cli.LOG, 'debug', mock.Mock())
    def test_show_lv_detail(self):
        self._test_show_command(
            self.cli_data.get_fake_show_lv_detail(),
            self.cli_data.get_test_show_lv_detail(),
            cli.ShowLV, '-l')

    @mock.patch.object(cli.LOG, 'debug', mock.Mock())
    def test_show_lv_tier(self):
        self._test_show_command(
            self.cli_data.get_fake_show_lv_tier(),
            self.cli_data.get_test_show_lv_tier(),
            cli.ShowLV, 'tier')

    @mock.patch.object(cli.LOG, 'debug', mock.Mock())
    def test_show_device(self):
        self._test_show_command(
            self.cli_data.get_fake_show_device(),
            self.cli_data.get_test_show_device(),
            cli.ShowDevice)

    @mock.patch.object(cli.LOG, 'debug', mock.Mock())
    def test_show_channel(self):
        self._test_show_command(
            self.cli_data.get_fake_show_channel(),
            self.cli_data.get_test_show_channel(),
            cli.ShowChannel)

    @mock.patch.object(cli.LOG, 'debug', mock.Mock())
    def test_show_channel_with_r_model(self):
        self._test_show_command(
            self.cli_data.get_fake_show_channel_r_model(),
            self.cli_data.get_test_show_channel_r_model(),
            cli.ShowChannel)

    @mock.patch.object(cli.LOG, 'debug', mock.Mock())
    def test_show_map(self):
        self._test_show_command(
            self.cli_data.get_fake_show_map(),
            self.cli_data.get_test_show_map(),
            cli.ShowMap)

    @mock.patch.object(cli.LOG, 'debug', mock.Mock())
    def test_show_license(self):
        self._test_show_command(
            self.cli_data.get_fake_show_license(),
            self.cli_data.get_test_show_license_full(),
            cli.ShowLicense)

    @mock.patch.object(cli.LOG, 'debug', mock.Mock())
    def test_show_replica_detail(self):
        self._test_show_command(
            self.cli_data.get_fake_show_replica_detail(),
            self.cli_data.get_test_show_replica_detail(),
            cli.ShowReplica, '-l')

    @mock.patch.object(cli.LOG, 'debug', mock.Mock())
    def test_show_wwn(self):
        self._test_show_command(
            self.cli_data.get_fake_show_wwn(),
            self.cli_data.get_test_show_wwn(),
            cli.ShowWWN)

    @mock.patch.object(cli.LOG, 'debug', mock.Mock())
    def test_show_iqn(self):
        self._test_show_command(
            self.cli_data.get_fake_show_iqn(),
            self.cli_data.get_test_show_iqn(),
            cli.ShowIQN)

    @mock.patch.object(cli.LOG, 'debug', mock.Mock())
    def test_show_host(self):
        self._test_show_command(
            self.cli_data.get_fake_show_host(),
            self.cli_data.get_test_show_host(),
            cli.ShowHost)



# Copyright (c) 2016 Huawei Technologies Co., Ltd.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

import ddt

from oslo_serialization import jsonutils
import webob

from cinder.api import microversions as mv
from cinder.api.v3 import router as router_v3
from cinder.backup import api as backup_api
from cinder import context
from cinder import objects
from cinder import test
from cinder.tests.unit.api import fakes
from cinder.tests.unit.backup import fake_backup
from cinder.tests.unit import fake_constants as fake


def fake_backup_get(*args, **kwargs):
    ctx = context.RequestContext(fake.USER_ID, fake.PROJECT_ID, False)
    bak = {
        'id': fake.BACKUP_ID,
        'project_id': fake.PROJECT_ID,
        'user_id': fake.USER_ID,
    }
    return fake_backup.fake_backup_obj(ctx, **bak)


def fake_backup_get_all(*args, **kwargs):
    return objects.BackupList(objects=[fake_backup_get()])


def app():
    # no auth, just let environ['cinder.context'] pass through
    api = router_v3.APIRouter()
    mapper = fakes.urlmap.URLMap()
    mapper['/v3'] = api
    return mapper


@ddt.ddt
class BackupProjectAttributeTest(test.TestCase):

    def setUp(self):
        super(BackupProjectAttributeTest, self).setUp()
        self.mock_object(backup_api.API, 'get', fake_backup_get)
        self.mock_object(backup_api.API, 'get_all', fake_backup_get_all)

    def _send_backup_request(self, ctx, detail=False,
                             version=mv.BACKUP_PROJECT):
        req = None
        if detail:
            req = webob.Request.blank(('/v3/%s/backups/detail'
                                       % fake.PROJECT_ID))
        else:
            req = webob.Request.blank('/v3/%s/backups/%s' % (fake.PROJECT_ID,
                                                             fake.BACKUP_ID))
        req.method = 'GET'
        req.environ['cinder.context'] = ctx
        req.headers = mv.get_mv_header(version)
        req.api_version_request = mv.get_api_version(version)
        res = req.get_response(app())

        if detail:
            return jsonutils.loads(res.body)['backups']
        return jsonutils.loads(res.body)['backup']

    @ddt.data(True, False)
    def test_get_backup_with_project(self, is_admin):
        ctx = context.RequestContext(fake.USER2_ID, fake.PROJECT_ID, is_admin)
        bak = self._send_backup_request(ctx)
        if is_admin:
            self.assertEqual(fake.PROJECT_ID,
                             bak['os-backup-project-attr:project_id'])
        else:
            self.assertNotIn('os-backup-project-attr:project_id', bak)

    @ddt.data(True, False)
    def test_list_detail_backups_with_project(self, is_admin):
        ctx = context.RequestContext(fake.USER2_ID, fake.PROJECT_ID, is_admin)
        baks = self._send_backup_request(ctx, detail=True)
        if is_admin:
            self.assertEqual(fake.PROJECT_ID,
                             baks[0]['os-backup-project-attr:project_id'])
        else:
            self.assertNotIn('os-backup-project-attr:project_id', baks[0])

    def test_get_backup_under_allowed_api_version(self):
        ctx = context.RequestContext(fake.USER2_ID, fake.PROJECT_ID, True)
        bak = self._send_backup_request(
            ctx, version=mv.get_prior_version(mv.BACKUP_PROJECT))
        self.assertNotIn('os-backup-project-attr:project_id', bak)

    @ddt.data(True, False)
    def test_get_backup_with_user_id(self, is_admin):
        ctx = context.RequestContext(fake.USER_ID, fake.PROJECT_ID, is_admin)
        bak = self._send_backup_request(ctx,
                                        version=mv.BACKUP_PROJECT_USER_ID)
        if is_admin:
            self.assertEqual(fake.USER_ID, bak['user_id'])
        else:
            self.assertNotIn('user_id', bak)

    @ddt.data(True, False)
    def test_list_detail_backups_with_user_id(self, is_admin):
        ctx = context.RequestContext(fake.USER_ID, fake.PROJECT_ID, is_admin)
        baks = self._send_backup_request(ctx, detail=True,
                                         version=mv.BACKUP_PROJECT_USER_ID)
        if is_admin:
            self.assertEqual(fake.USER_ID,
                             baks[0]['user_id'])
        else:
            self.assertNotIn('user_id', baks[0])

    def test_get_backup_user_id_before_microversion_v356(self):
        ctx = context.RequestContext(fake.USER_ID, fake.PROJECT_ID, True)
        bak = self._send_backup_request(
            ctx, version=mv.get_prior_version(mv.BACKUP_PROJECT_USER_ID))
        self.assertNotIn('user_id', bak)



# Copyright (c) 2017 Huawei Technologies Co., Ltd.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_policy import policy

from cinder.policies import base


EXTEND_POLICY = "volume:extend"
EXTEND_ATTACHED_POLICY = "volume:extend_attached_volume"
REVERT_POLICY = "volume:revert_to_snapshot"
RESET_STATUS = "volume_extension:volume_admin_actions:reset_status"
RETYPE_POLICY = "volume:retype"
UPDATE_READONLY_POLICY = "volume:update_readonly_flag"
FORCE_DELETE_POLICY = "volume_extension:volume_admin_actions:force_delete"
FORCE_DETACH_POLICY = "volume_extension:volume_admin_actions:force_detach"
UPLOAD_PUBLIC_POLICY = "volume_extension:volume_actions:upload_public"
UPLOAD_IMAGE_POLICY = "volume_extension:volume_actions:upload_image"
MIGRATE_POLICY = "volume_extension:volume_admin_actions:migrate_volume"
MIGRATE_COMPLETE_POLICY = \
    "volume_extension:volume_admin_actions:migrate_volume_completion"
DETACH_POLICY = "volume_extension:volume_actions:detach"
ATTACH_POLICY = "volume_extension:volume_actions:attach"
BEGIN_DETACHING_POLICY = "volume_extension:volume_actions:begin_detaching"
UNRESERVE_POLICY = "volume_extension:volume_actions:unreserve"
RESERVE_POLICY = "volume_extension:volume_actions:reserve"
ROLL_DETACHING_POLICY = "volume_extension:volume_actions:roll_detaching"
TERMINATE_POLICY = "volume_extension:volume_actions:terminate_connection"
INITIALIZE_POLICY = "volume_extension:volume_actions:initialize_connection"

volume_action_policies = [
    policy.DocumentedRuleDefault(
        name=EXTEND_POLICY,
        check_str=base.RULE_ADMIN_OR_OWNER,
        description="Extend a volume.",
        operations=[
            {
                'method': 'POST',
                'path': '/volumes/{volume_id}/action (os-extend)'
            }
        ]),
    policy.DocumentedRuleDefault(
        name=EXTEND_ATTACHED_POLICY,
        check_str=base.RULE_ADMIN_OR_OWNER,
        description="Extend a attached volume.",
        operations=[
            {
                'method': 'POST',
                'path': '/volumes/{volume_id}/action (os-extend)'
            }
        ]),
    policy.DocumentedRuleDefault(
        name=REVERT_POLICY,
        check_str=base.RULE_ADMIN_OR_OWNER,
        description="Revert a volume to a snapshot.",
        operations=[
            {
                'method': 'POST',
                'path': '/volumes/{volume_id}/action (revert)'
            }
        ]),
    policy.DocumentedRuleDefault(
        name=RESET_STATUS,
        check_str=base.RULE_ADMIN_API,
        description="Reset status of a volume.",
        operations=[
            {
                'method': 'POST',
                'path': '/volumes/{volume_id}/action (os-reset_status)'
            }
        ]),
    policy.DocumentedRuleDefault(
        name=RETYPE_POLICY,
        check_str=base.RULE_ADMIN_OR_OWNER,
        description="Retype a volume.",
        operations=[
            {
                'method': 'POST',
                'path': '/volumes/{volume_id}/action (os-retype)'
            }
        ]),
    policy.DocumentedRuleDefault(
        name=UPDATE_READONLY_POLICY,
        check_str=base.RULE_ADMIN_OR_OWNER,
        description="Update a volume's readonly flag.",
        operations=[
            {
                'method': 'POST',
                'path': '/volumes/{volume_id}/action (os-update_readonly_flag)'
            }
        ]),
    policy.DocumentedRuleDefault(
        name=FORCE_DELETE_POLICY,
        check_str=base.RULE_ADMIN_API,
        description="Force delete a volume.",
        operations=[
            {
                'method': 'POST',
                'path': '/volumes/{volume_id}/action (os-force_delete)'
            }
        ]),
    policy.DocumentedRuleDefault(
        name=UPLOAD_PUBLIC_POLICY,
        check_str=base.RULE_ADMIN_API,
        description="Upload a volume to image with public visibility.",
        operations=[
            {
                'method': 'POST',
                'path': '/volumes/{volume_id}/action (os-volume_upload_image)'
            }
        ]),
    policy.DocumentedRuleDefault(
        name=UPLOAD_IMAGE_POLICY,
        check_str=base.RULE_ADMIN_OR_OWNER,
        description="Upload a volume to image.",
        operations=[
            {
                'method': 'POST',
                'path': '/volumes/{volume_id}/action (os-volume_upload_image)'
            }
        ]),
    policy.DocumentedRuleDefault(
        name=FORCE_DETACH_POLICY,
        check_str=base.RULE_ADMIN_API,
        description="Force detach a volume.",
        operations=[
            {
                'method': 'POST',
                'path': '/volumes/{volume_id}/action (os-force_detach)'
            }
        ]),
    policy.DocumentedRuleDefault(
        name=MIGRATE_POLICY,
        check_str=base.RULE_ADMIN_API,
        description="migrate a volume to a specified host.",
        operations=[
            {
                'method': 'POST',
                'path': '/volumes/{volume_id}/action (os-migrate_volume)'
            }
        ]),
    policy.DocumentedRuleDefault(
        name=MIGRATE_COMPLETE_POLICY,
        check_str=base.RULE_ADMIN_API,
        description="Complete a volume migration.",
        operations=[{
            'method': 'POST',
            'path':
                '/volumes/{volume_id}/action (os-migrate_volume_completion)'}
        ]),
    policy.DocumentedRuleDefault(
        name=INITIALIZE_POLICY,
        check_str=base.RULE_ADMIN_OR_OWNER,
        description="Initialize volume attachment.",
        operations=[{
            'method': 'POST',
            'path':
                '/volumes/{volume_id}/action (os-initialize_connection)'}
        ]),
    policy.DocumentedRuleDefault(
        name=TERMINATE_POLICY,
        check_str=base.RULE_ADMIN_OR_OWNER,
        description="Terminate volume attachment.",
        operations=[{
            'method': 'POST',
            'path':
                '/volumes/{volume_id}/action (os-terminate_connection)'}
        ]),
    policy.DocumentedRuleDefault(
        name=ROLL_DETACHING_POLICY,
        check_str=base.RULE_ADMIN_OR_OWNER,
        description="Roll back volume status to 'in-use'.",
        operations=[{
            'method': 'POST',
            'path':
                '/volumes/{volume_id}/action (os-roll_detaching)'}
        ]),
    policy.DocumentedRuleDefault(
        name=RESERVE_POLICY,
        check_str=base.RULE_ADMIN_OR_OWNER,
        description="Mark volume as reserved.",
        operations=[{
            'method': 'POST',
            'path':
                '/volumes/{volume_id}/action (os-reserve)'}
        ]),
    policy.DocumentedRuleDefault(
        name=UNRESERVE_POLICY,
        check_str=base.RULE_ADMIN_OR_OWNER,
        description="Unmark volume as reserved.",
        operations=[{
            'method': 'POST',
            'path':
                '/volumes/{volume_id}/action (os-unreserve)'}
        ]),
    policy.DocumentedRuleDefault(
        name=BEGIN_DETACHING_POLICY,
        check_str=base.RULE_ADMIN_OR_OWNER,
        description="Begin detach volumes.",
        operations=[{
            'method': 'POST',
            'path':
                '/volumes/{volume_id}/action (os-begin_detaching)'}
        ]),
    policy.DocumentedRuleDefault(
        name=ATTACH_POLICY,
        check_str=base.RULE_ADMIN_OR_OWNER,
        description="Add attachment metadata.",
        operations=[{
            'method': 'POST',
            'path':
                '/volumes/{volume_id}/action (os-attach)'}
        ]),
    policy.DocumentedRuleDefault(
        name=DETACH_POLICY,
        check_str=base.RULE_ADMIN_OR_OWNER,
        description="Clear attachment metadata.",
        operations=[{
            'method': 'POST',
            'path':
                '/volumes/{volume_id}/action (os-detach)'}
        ]),
]


def list_rules():
    return volume_action_policies
